#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import weakref

import wx

from .wx_widget_component import WXWidgetComponent
from .wx_menu import wxEnamlMenu

from ...components.menu_bar import AbstractTkMenuBar


class wxEnamlMenuBar(wx.MenuBar):
    """ A wx.MenuBar subclass which adapts the wx menu bar api to work 
    with the menu semantics of Enaml.

    """
    def __init__(self, parent, *args, **kwargs):
        """ Initialize a wxEnamlMenuBar.

        Parameters
        ----------
        parent : wx.Frame
            An instance of wx.Frame to which this menu bar is attached.

        *args
            Additional positional arguments to pass to the wx.MenuBar
            constructor.
        
        **kwargs
            Additional keyword arguments to pass to the wx.MenuBar
            constructor.

        """
        if not isinstance(parent, wx.Frame):
            msg = ('The parent of a wxCustomMenuBar must be an instance of '
                   'wx.Frame. Got %s instead' % parent)
            raise TypeError(msg)
        super(wxEnamlMenuBar, self).__init__(*args, **kwargs)
        self._parent = weakref.ref(parent)
        self._enabled = True

    def GetParent(self):
        """ Returns the parent of this menu bar, which will be a wxFrame,
        or None if the frame has already been garbage collected.

        """
        return self._parent()

    def Append(self, menu, title=''):
        """ Appends the given menu to the menu bar with the given title.

        Parameters
        ----------
        menu : wxEnamlMenu
            The instance of wxEnamlMenu which should be appended to the
            menu bar.
        
        title : string, optional
            The title that should be used for the menu. If not provided
            or an empty string, a unique mangled title will be created.

        """
        idx = self.GetMenuCount()
        self.Insert(idx, menu, title)

    def Insert(self, idx, menu, title=''):
        """ Appends the given menu to the menu bar with the given title.

        Parameters
        ----------
        idx : int
            The index at which to insert the menu.

        menu : wxEnamlMenu
            The instance of wxEnamlMenu which should be appended to the
            menu bar.
        
        title : string, optional
            The title that should be used for the menu. If not provided
            or an empty string, a unique mangled title will be created.

        """
        if idx < 0 or idx > self.GetMenuCount():
            msg = 'Cannot insert menu. Index %s is out of range.'
            raise ValueError(msg % idx)

        if not isinstance(menu, wxEnamlMenu):
            msg = ('Can only insert instances of wxEnamlMenu to a '
                   'wxEnamlMenuBar. Got %s instead.')
            raise TypeError(msg % menu)

        # wx will raise a C++ exception if the title is an empty string.
        # In such cases, we mangle a unique title for the menu.
        if not title:
            title = 'Menu_' + self.GetMenuCount()
        
        super(wxEnamlMenuBar, self).Insert(idx, menu, title)

        # After adding the menu, we need to check whether we should
        # disable the menu based on its enabled state. This api is
        # a bit weird since wx forces us to set the enabledness of
        # the menu through the menubar object, so the repsonsibility
        # of that is shared between here and the Menu object itself.
        if not self.IsEnabled() or not menu.IsEnabled():
            self.EnableTop(self.GetMenuCount() - 1, False)

    def RemoveAll(self):
        """ Removes all the menus from the menu bar. The menus are not
        destroyed.

        """
        while self.GetMenuCount():
            self.Remove(0)
        
    def IsEnabled(self):
        """ Returns whether or not the menu bar is enabled. Note that 
        this overrides a parent class method of the same name but with
        a different call signature and different semantics.

        """
        return self._enabled

    def Enable(self, enable):
        """ Enables or disables all of the underlying children. Note 
        that this overrides a parent class method of the same name 
        but with a different call signature and different semantics.

        """
        menus = [item[0] for item in self.GetMenus()]
        for idx, menu in enumerate(menus):
            if enable:
                # Only enable the menu if its underlying enabled state
                # is True. This cooperation is required because of the
                # funky menu api of wx. The menu itself also has logic
                # which will toggle its enabled state.
                if menu.IsEnabled():
                    self.EnableTop(idx, True)
            else:
                self.EnableTop(idx, False)
        self._enabled = enable

    def Destroy(self):
        """ An overridden destructor method which removes itself from
        the parent before invoking the actual destructor.

        """
        if self:
            parent = self._parent()
            if parent is not None:
                if parent.GetMenuBar() == self:
                    parent.SetMenuBar(None)
            super(wxEnamlMenuBar, self).Destroy()


class WXMenuBar(WXWidgetComponent, AbstractTkMenuBar):
    """ A Wx implementation of a MenuBar.

    """
    def create(self, parent):
        """ Creates the underlying wxMenuBar.

        """
        self.widget = wxEnamlMenuBar(parent)
        # This makes sure we avoid IsAttached assertion errors on 
        # Windows during initalization. The menu bar must be attached
        # before calling various methods on it, which happens during
        # initialization.
        parent.SetMenuBar(self.widget)

    def initialize(self):
        """ Initializes the wxMenuBar.

        """
        super(WXMenuBar, self).initialize()
        self.update_menus()
    
    #--------------------------------------------------------------------------
    # Change Handlers 
    #--------------------------------------------------------------------------
    def shell_menus_changed(self):
        """ The change handler for the 'menus' attribute on the shell
        object.

        """
        self.update_menus()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def update_menus(self):
        """ Updates the menu bar with the child menu objects.

        """
        # We can't reason reliably on the proper ordering of menus
        # in the menu bar because of the variety of possible uses
        # of the Include component. Instead, we just remove the 
        # existing menus, and then add the desired menus.
        widget = self.widget
        widget.RemoveAll()
        for menu in self.shell_obj.menus:
            widget.Append(menu.toolkit_widget, menu.title)
        
        # XXX - This is a hack to workaround an issue where the menu bar
        # does not properly repaint itself on Windows after inserting
        # new menus. Forcing an Update or a parent Frame refresh does
        # not solve the problem. So far, this is the only thing I have
        # found which works and causes the menu bar to redraw itself.
        frame = widget.GetParent()
        if frame:
            frame.Freeze()
            frame.SetMenuBar(None)
            frame.SetMenuBar(widget)
            frame.Thaw()




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance

from .base_widget_component import (
    BaseWidgetComponent, AbstractTkBaseWidgetComponent,
)

from ..enums import SelectionMode, SelectionBehavior
from ..core.trait_types import EnamlEvent


class AbstractTkBaseSelectionModel(AbstractTkBaseWidgetComponent):
    """ The abstract toolkit BaseSelectionModel interface.

    """
    @abstractmethod
    def clear(self):
        """ Clear the selection and the current index.

        """
        raise NotImplementedError

    @abstractmethod
    def get_current_index(self):
        """ Return the current ModelIndex or None if there is not one.

        """
        raise NotImplementedError

    @abstractmethod
    def set_current_index(self, index):
        """ Set the current ModelIndex.

        """
        raise NotImplementedError

    @abstractmethod
    def set_selection(self, selection, command='clear_select'):
        """ Set the current selection.

        """
        raise NotImplementedError

    @abstractmethod
    def get_selection(self):
        """ Get the current selection.

        """
        raise NotImplementedError

    @abstractmethod
    def set_selection_mode(self, selection_mode):
        raise NotImplementedError

    @abstractmethod
    def set_selection_behavior(self, selection_behavior):
        raise NotImplementedError


class BaseSelectionModel(BaseWidgetComponent):
    """ The base class for item selection models.

    """
    #: Updated when the current ModelIndex changes. Gets a 2-tuple of
    #: (old ModelIndex, new ModelIndex)
    current_event = EnamlEvent

    #: Updated when the current selection changes. Gets a 2-tuple of
    #: (deleted items, added items). Each selection is a list of
    #: (top_left ModelIndex, bottom_right ModelIndex) tuples specifying
    #: rectangular ranges of selected cells.
    selection_event = EnamlEvent

    #: The selection mode.
    selection_mode = SelectionMode

    #: What kinds of things can be selected.
    selection_behavior = SelectionBehavior

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkBaseSelectionModel)

    def clear(self):
        """ Clear the selection and the current index.

        """
        self.abstract_obj.clear()

    def set_current_index(self, index):
        """ Set the current ModelIndex.

        This is the cell used for keyboard focus and is usually set when
        the user clicks a cell. This may be independent of the selection. 
        It is frequently rendered with a dashed border.

        Parameters
        ----------
        index : ModelIndex
            The index to make current.

        """
        self.abstract_obj.set_current_index(index)

    def get_current_index(self):
        """ Get the current ModelIndex.

        """
        return self.abstract_obj.get_current_index()

    def set_selection(self, selection, command='clear_select'):
        """ Set the current selection.

        Parameters
        ----------
        selection : list of (ModelIndex, ModelIndex) tuples
            Each tuple is an inclusive range specifying a bounding box
            for a given selection range.
            
        command : single or sequence of SelectionCommand, optional
            Exactly what action to perform given these selection ranges. 
            See the `SelectionCommand` documentation for the complete 
            set. Sequences of SelectionCommands will be treated as 
            binary-ORing the flags.
        
        """
        self.abstract_obj.set_selection(selection, command)

    def get_selection(self):
        """ Get the current selection.

        Returns
        -------
        selection : list of (ModelIndex, ModelIndex) tuples
            Each tuple is an inclusive range specifying a bounding box 
            for a given selection range.

        """
        return self.abstract_obj.get_selection()

    def _selection_mode_changed(self, new):
        if self.abstract_obj is not None:
            self.abstract_obj.set_selection_mode(new)

    def _selection_behavior_changed(self, new):
        if self.abstract_obj is not None:
            self.abstract_obj.set_selection_behavior(new)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from unittest import expectedFailure
import warnings

import wx

from .wx_test_assistant import WXTestAssistant, skip_nonwindows

from .. import field


warnings.simplefilter('ignore')


@skip_nonwindows
class TestWxField(WXTestAssistant, field.TestField):
    """ WXField tests. 

    """
    def get_value(self, widget):
        """ Get the visible text of a field.

        """
        self.process_wx_events(self.app)
        return widget.GetValue()

    def edit_text(self, widget, text):
        """ Simulate typing in a field.

        """
        widget.WriteText(text)
        self.send_wx_event(widget, wx.EVT_TEXT)
        self.process_wx_events(self.app)

    def change_text(self, widget, text):
        """ Change text programmatically, rather than "edit" it.

        """
        widget.ChangeValue(text)
        self.process_wx_events(self.app)

    def set_cursor_position(self, widget, index):
        """ Set the cursor at a specific position.

        """
        widget.SetInsertionPoint(index)
        self.process_wx_events(self.app)

    def get_cursor_position(self, widget):
        """ Get the cursor position.

        """
        self.process_wx_events(self.app)
        return widget.GetInsertionPoint()

    def get_password_mode(self, widget):
        """ Get the password mode status of the widget.

        Currently WXField only supports `password` and normal`.

        """
        # FIXME: specifically checking if we are using WX and the component
        # is set to 'silent' so that we can set the test to expected failure
        # we use this ugly hack.
        @expectedFailure
        def silent_password_mode(self):
            self.fail('Currently WXField only supports `password` and normal`.')

        component = self.component
        if component.password_mode == 'silent':
            silent_password_mode()

        self.process_wx_events(self.app)
        if widget.HasFlag(wx.TE_PASSWORD):
            return 'password'
        else:
            return 'normal'

    def get_selected_text(self, widget):
        """ Get the currently-selected text from a field.

        """
        self.process_wx_events(self.app)
        return widget.GetStringSelection()

    def press_return(self, widget):
        """ Simulate a press of the 'Return' key.

        """
        self.send_wx_event(widget, wx.EVT_TEXT_ENTER)
        self.process_wx_events(self.app)

    def gain_focus_if_needed(self, widget):
        """ Have the widget gain focus if required for the tests.

        """
        # wx doesn't seem to handle anything on the widget properly 
        # if the widget's not visible or doesn't have focus. So, in order 
        # for these tests to run, we need to gain focus on the widget. This
        # is not required in normal circumstances. Further, we can't get
        # widget to SetFocus before the value has been set, so we need
        # to "re-initialize" the widget it. This is really terrible and
        # it would be good to know which part of wx is broken so this
        # can be fixed.
        widget.SetFocus()
        val = self.component.value
        self.component.value = ''
        self.component.value = val




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_window import WXWindow

from ...components.dialog import AbstractTkDialog


DIALOG_RETCODE_MAP = {
    wx.ID_OK: 'accepted',
    wx.ID_CANCEL: 'rejected'
}


class WXDialogSizer(wx.PySizer):
    """ A custom wx Sizer for use in the WXDialog. This sizers expands
    its child to fit the allowable space, regardless of the settings on
    the child settings. This is similar to how central widgets behave 
    in a WXWindow. 

    There can only be one widget in this sizer at a time and it should
    be added via the .Add(...) method. Old items will be removed 
    automatically (but not destroyed).

    """
    def __init__(self, *args, **kwargs):
        super(WXDialogSizer, self).__init__(*args, **kwargs)
        self._widget = None

    def Add(self, widget):
        """ Adds the given widget to the sizer, removing the old widget
        if present. The old widget is not destroyed.

        """
        self.Clear(deleteWindows=False)
        self._widget = widget
        return super(WXDialogSizer, self).Add(widget)

    def CalcMin(self):
        """ Returns the minimum size for the children this sizer is 
        managing. Since the size of the Dialog is managed externally,
        this always returns (-1, -1).

        """
        return (-1, -1)
    
    def RecalcSizes(self):
        """ Resizes the child to fit the available space of the scroll
        area.

        """
        widget = self._widget
        if widget:
            widget.SetSize(self.GetSize())


class WXDialog(WXWindow, AbstractTkDialog):
    """ A wxPython implementation of a Dialog.

    WXDialog uses a wx.Dialog to create a simple top-level dialog.

    """
    #---------------------------------------------------------------------------
    # Setup methods
    #---------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.Dialog control.

        """
        style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
        self.widget = wx.Dialog(parent, style=style)
        self.widget.SetSizer(WXDialogSizer())

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def accept(self):
        """ Close the dialog and set the result to 'accepted'.

        """
        self.widget.EndModal(wx.ID_OK)

    def reject(self):
        """ Close the dialog and set the result to 'rejected'.

        """
        self.widget.EndModal(wx.ID_CANCEL)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_central_widget(self, central_widget):
        """ Sets the central widget in the window with the given value.

        """
        # It's possible for the central widget component to be None.
        # This must be allowed since the central widget may be generated
        # by an Include component, in which case it will not exist 
        # during initialization. However, we must have a central widget
        # for the Dialog, and so we just fill it with a dummy widget.
        central_widget = self.shell_obj.central_widget
        if central_widget is None:
            child_widget = wx.Panel(self.widget)
        else:
            child_widget = central_widget.toolkit_widget
        self.widget.GetSizer().Add(child_widget)

    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def set_visible(self, visible):
        """ Overridden from the parent class to properly launch and close 
        the dialog.

        """
        widget = self.widget
        shell = self.shell_obj
        if visible:
            shell._active = True
            shell.opened()
            # wx cannot distinguish between app modal and 
            # window modal, so we only get one kind.
            retcode = widget.ShowModal()
        else:
            retcode = wx.ID_CANCEL
        self._handle_retcode(retcode)
    
    #--------------------------------------------------------------------------
    # Auxiliary Methods 
    #--------------------------------------------------------------------------
    def _handle_retcode(self, retcode):
        """ Destroys the dialog, fires events, and set status attributes.

        """
        shell = self.shell_obj
        result = DIALOG_RETCODE_MAP[retcode]
        shell._result = result
        shell._active = False
        shell.closed(result)
        # Explicitly Destroy the dialog or the wxApp won't properly exit.
        # We can't simply destroy the shell object since the user may
        # still need something from it.
        widget = self.widget
        if widget:
            widget.Destroy()        




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import TraitError

from .enaml_test_case import EnamlTestCase, required_method


class TestEvents(object):
    """ Events required by the testcase.

    """
    #: The left button is pressed
    PRESSED = 0x0

    #: The left button is released
    RELEASED = 0x1

    #: The thumb is moved one step up
    STEP_UP = 0x2

    #: The thumb is moved one step down
    STEP_DOWN = 0x3

    #: The thumb is moved one page up
    PAGE_UP = 0x4

    #: The thumb is moved one page down
    PAGE_DOWN = 0x5

    #: The thumb is moved to home
    HOME = 0x6

    #: The thumb is moved to the end
    END = 0x7


class TestSlider(EnamlTestCase):
    """ Logic for testing sliders.


    Tooklit testcases need to provide the following methods

    Abstract Methods
    ----------------
    get_value(self, widget)
        Get the Slider's value.

    get_tick_interval(self, widget)
        Get the Slider's tick_interval value.

    get_tick_position(self, widget)
        Get the Slider's tick position style.

    get_orientation(self, widget)
        Get the Slider's orientation.

    get_single_step(self, widget)
        Get the Slider's single step value.

    get_page_step(self, widget)
        Get the Slider's page step value.

    get_tracking(self, widget)
        Get the Slider's tracking status.

    send_event(self, widget, event)
        Send an event to the Slider programmatically.

    """

    def setUp(self):
        """ Set up before the spin box tests.

        """

        enaml_source = """
enamldef MainView(MainWindow):
    attr events
    Slider:
        name = 'slider'
        moved :: events.append(('moved', event.new))
        pressed :: events.append('pressed')
        released :: events.append('released')
"""

        self.events = []
        self.view = self.parse_and_create(enaml_source, events=self.events)
        self.component = self.component_by_name(self.view, 'slider')
        self.widget = self.component.toolkit_widget

    def test_initial_attributes(self):
        """ Compare the Enaml Slider's attributes with its toolkit widget.

        """
        component = self.component

        self.assertFalse(component.error)
        self.assertIsNone(component.exception)

        self.assertEnamlInSync(component, 'value', 0)
        self.assertEnamlInSync(component, 'minimum', 0)
        self.assertEnamlInSync(component, 'maximum', 100)
        self.assertEnamlInSync(component, 'tick_interval', 10)
        self.assertEnamlInSync(component, 'tick_position', 'bottom')
        self.assertEnamlInSync(component, 'orientation', 'horizontal')
        self.assertEnamlInSync(component, 'single_step', 1)
        self.assertEnamlInSync(component, 'page_step', 10)
        self.assertEqual(self.events, [])


    def test_tracking_attribute(self):
        """ Test accesing tracking attribute.

        """
        component = self.component

        self.assertEnamlInSync(component, 'tracking', True)

        component.tracking = False
        self.assertEnamlInSync(component, 'tracking', False)
        self.assertEqual(self.events, [])


    def test_change_maximum(self):
        """ Test changing the maximum range of the slider.

        """
        component = self.component
        component.maximum = 200
        self.assertEnamlInSync(component, 'maximum', 200)

    def test_change_minimum(self):
        """ Test changing the minimum range of the slider.

        """
        component = self.component
        component.value = 70
        component.minimum = 50
        self.assertEnamlInSync(component, 'minimum', 50)
        self.assertEqual(self.events, [])

    def test_change_maximum_invalid(self):
        """ Test attempting to set an invalid value to the maximum range
        of the slider.

        """
        component = self.component
        with self.assertRaises(TraitError):
            component.maximum = -23
        component.minimum = 50
        with self.assertRaises(TraitError):
            component.maximum = 23
        self.assertEqual(self.events, [])

    def test_change_minimum_invalid(self):
        """ Test attempting to set an invalid value to the minimum range
        of the slider.

        """
        component = self.component
        with self.assertRaises(TraitError):
            component.minimum = -13
        with self.assertRaises(TraitError):
            component.minimum = 135
        self.assertEqual(self.events, [])

    def test_change_maximum_and_value(self):
        """ Test setting maximum while the value is out of range.

        """
        component = self.component
        component.value = 70
        self.assertEnamlInSync(component, 'value', 70)
        component.maximum = 50
        self.assertEnamlInSync(component, 'value', 50)
        self.assertEqual(self.events, [])

    def test_change_minimum_and_value(self):
        """ Test setting minimum while the value is out of range.

        """
        component = self.component
        component.value = 30
        component.minimum = 50
        self.assertEnamlInSync(component, 'value', 50)
        self.assertEqual(self.events, [])

    def test_value_change(self):
        """ Test changing the value programmaticaly.

        """
        component = self.component
        component.value = 1
        self.assertEnamlInSync(component, 'value', 1)
        component.value = 34
        self.assertEnamlInSync(component, 'value', 34)
        self.assertEqual(self.events, [])

    def test_invalid_value_change(self):
        """ Test changing the position with the an invalid value

        when invalid, check that it has not changed the values and the
        errors are updated.

        """
        component = self.component

        with self.assertRaises(TraitError):
            component.value = -2
        with self.assertRaises(TraitError):
            component.value = 120

    def test_orientation_setting(self):
        """ Test changing the widget orientation

        """
        component = self.component

        self.component.orientation = 'vertical'
        self.assertEnamlInSync(component, 'orientation', 'vertical')

        self.component.orientation = 'horizontal'
        self.assertEnamlInSync(component, 'orientation', 'horizontal')

    def test_tick_position_setting(self):
        """ Test changing the widget tickposition

        """
        component = self.component

        component.tick_position = 'bottom'
        self.assertEnamlInSync(component, 'tick_position', 'bottom')

        component.tick_position = 'top'
        self.assertEnamlInSync(component, 'tick_position', 'top')

        component.tick_position = 'both'
        self.assertEnamlInSync(component, 'tick_position', 'both')

        self.component.orientation = 'vertical'

        component.tick_position = 'left'
        self.assertEnamlInSync(component, 'tick_position', 'left')

        component.tick_position = 'right'
        self.assertEnamlInSync(component, 'tick_position', 'right')

        component.tick_position = 'both'
        self.assertEnamlInSync(component, 'tick_position', 'both')

        component.tick_position = 'no_ticks'
        self.assertEnamlInSync(component, 'tick_position', 'no_ticks')

    def test_incompatible_tick_position(self):
        """ Test that changing tick position is addapted if the orientation
        is not compatible.

        This is in sync with how the QSlider behaves.

        """
        component = self.component

        component.tick_position = 'left'
        self.assertEnamlInSync(component, 'tick_position', 'top')

        component.tick_position = 'right'
        self.assertEnamlInSync(component, 'tick_position', 'bottom')

        self.component.orientation = 'vertical'

        component.tick_position = 'bottom'
        self.assertEnamlInSync(component, 'tick_position', 'right')

        component.tick_position = 'top'
        self.assertEnamlInSync(component, 'tick_position', 'left')

    def test_changing_orientaion_tick_policy(self):
        """ Test that the ticks follow the orientation changes

        """
        component = self.component

        component.tick_position = 'bottom'
        self.component.orientation = 'vertical'
        self.assertEnamlInSync(component, 'tick_position', 'right')
        component.tick_position = 'left'
        self.component.orientation = 'horizontal'
        self.assertEnamlInSync(component, 'tick_position', 'top')

    def test_pressing_the_thumb(self):
        """ Test firing events when the thumb is pressed down.

        """
        events = self.events
        component = self.component
        component.value = 50
        self.send_event(self.widget, TestEvents.PRESSED)
        self.assertEqual(['pressed'], events)

    # This fails on windows under wx because the underlying size of the
    # of the widget is not reliable. So the precomputed mouse
    # position for the mouse event which is sent can be wrong.
    def test_releasing_the_thumb(self):
        """ Test firing events when the thumb is released.

        """
        events = self.events
        component = self.component
        component.value = 50

        self.send_event(self.widget, TestEvents.RELEASED)
        self.assertEqual([], events)

        self.send_event(self.widget, TestEvents.PRESSED)
        self.send_event(self.widget, TestEvents.RELEASED)
        self.assertEqual(['pressed', 'released'], events)

    def test_moving_the_thumb_programmaticaly(self):
        """ Test firing events when the thumb is moved (programmatically).

        """
        component = self.component
        events = self.events
        component.value = 30
        self.assertEqual(events, [])
        self.assertEnamlInSync(component, 'value', 30)

    def test_move_to_home(self):
        """ Test firing events and value when the thumb is moved to home.

        """
        component = self.component
        events = self.events
        component.value = 50
        self.send_event(self.widget, TestEvents.HOME)
        self.assertEnamlInSync(component, 'value', 0)
        self.assertEqual(events, [])

    def test_move_to_end(self):
        """ Test firing events and value when the thumb is moved to end.

        """
        component = self.component
        events = self.events
        self.send_event(self.widget, TestEvents.END)
        self.assertEnamlInSync(component, 'value', 100)
        self.assertEqual(events, [])

    def test_move_down_by_one_step(self):
        """ Test firing events and value when the thumb is moved by one
        step down.

        """
        component = self.component
        events = self.events
        component.value = 50
        self.send_event(self.widget, TestEvents.STEP_DOWN)
        self.assertEnamlInSync(component, 'value', 49)
        self.assertEqual(events, [])

    def test_move_up_by_one_step(self):
        """ Test firing events and value when the thumb is moved by one
        step up.

        """
        component = self.component
        events = self.events
        component.value = 50
        self.send_event(self.widget, TestEvents.STEP_UP)
        self.assertEnamlInSync(component, 'value', 51)
        self.assertEqual(events, [])

    def test_move_down_by_one_page(self):
        """ Test firing events and value when the thumb is moved by one
        page down.

        """
        component = self.component
        events = self.events
        component.value = 50
        self.send_event(self.widget, TestEvents.PAGE_DOWN)
        self.assertEnamlInSync(component, 'value', 40)
        self.assertEqual(events, [])

    def test_move_up_by_one_page(self):
        """ Test firing events and value when the thumb is moved by one
        page up.

        """
        component = self.component
        events = self.events
        component.value = 50
        self.send_event(self.widget, TestEvents.PAGE_UP)
        self.assertEnamlInSync(component, 'value', 60)
        self.assertEqual(events, [])

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_value(self, widget):
        """ Get the Slider's value.

        """
        pass

    @required_method
    def get_minimum(self, widget):
        """ Get the Slider's minimum value.

        """
        pass

    @required_method
    def get_maximum(self, widget):
        """ Get the Slider's maximum value.

        """
        pass

    @required_method
    def get_tick_interval(self, widget):
        """ Get the Slider's tick_interval value.

        """
        pass

    @required_method
    def get_tick_position(self, widget):
        """ Get the Slider's tick position style.

        """
        pass

    @required_method
    def get_orientation(self, widget):
        """ Get the Slider's orientation.

        """
        pass

    @required_method
    def get_single_step(self, widget):
        """ Get the Slider's single step value.

        """
        pass

    @required_method
    def get_page_step(self, widget):
        """ Get the Slider's page step value.

        """
        pass

    @required_method
    def get_tracking(self, widget):
        """ Get the Slider's tracking status.

        """
        pass

    @required_method
    def send_event(self, widget, event):
        """ Send an event to the Slider programmatically.

        Arguments
        ---------
        widget :
            The widget to send the event to.

        event :
            The desired event to be proccessed.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import EnamlTestCase, required_method


class TestLabel(EnamlTestCase):
    """ Logic for testing labels.

    Tooklit testcases need to provide the following methods

    Abstract Methods
    ----------------
    get_text()
        Returns the label text from the tookit widget

    """

    def setUp(self):
        """ Set up label tests.

        """

        enaml_source = """
enamldef MainView(MainWindow):
    Label:
        name = 'label'
        text = 'foo'
"""

        self.view = self.parse_and_create(enaml_source)
        self.component = self.component_by_name(self.view, 'label')
        self.widget = self.component.toolkit_widget

    def test_initial_text(self):
        """ Test the initial text of a label.

        """
        self.assertEnamlInSync(self.component, 'text', 'foo')

    def test_text_changed(self):
        """ Change the text of the label.

        """

        self.component.text = 'bar'
        self.assertEnamlInSync(self.component, 'text', 'bar')

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_text(self, widget):
        """ Returns the label text from the tookit widget

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import EnamlTestCase, required_method


class TestWindow(EnamlTestCase):
    """ Logic for testing Windows.

    """
    def setUp(self):
        """ Set up Window tests.

        """

        enaml_source = """
enamldef MainView(MainWindow):
    name = 'window'
    title = 'foo'
"""

        self.view = self.parse_and_create(enaml_source)
        self.component = self.component_by_name(self.view, 'window')
        self.widget = self.component.toolkit_widget

    def test_initial_title(self):
        """ Test the initial title of a Window.

        """
        self.assertEnamlInSync(self.component, 'title', 'foo')

    def test_title_changed(self):
        """ Change the title of the Window.

        """
        self.component.title = 'bar'
        self.assertEnamlInSync(self.component, 'title', 'bar')

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_title(self, widget):
        """ Returns the title from the toolkit widget.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import TraitType

from .style_sheet import BoxedValue, StyleValue


NO_VALUE = object()


class _UserValue(BoxedValue):
    pass


class _DefaultValue(BoxedValue):
    pass


class StyleTrait(TraitType):

    def __init__(self, default_value=NO_VALUE, factory=None, **metadata):
        super(StyleTrait, self).__init__(**metadata)
        self._default_value_kw = default_value
        self._factory = factory

    def get(self, obj, name):
        attr = '_style_' + name
        try:
            boxed = obj.__dict__[attr]
        except KeyError:
            boxed = _DefaultValue(self._default_value(obj, name))
            obj.__dict__[attr] = boxed
        return boxed.value        

    def set(self, obj, name, value):
        # If we are being set with a StyleValue, then we only allow
        # the setting to occur if the current value is not a _UserValue
        attr = '_style_' + name
        try:
            old_boxed = obj.__dict__[attr]
        except KeyError:
            old_boxed = None
            old_value = NO_VALUE
        else:
            old_value = old_boxed.value

        if isinstance(value, StyleValue):
            if old_boxed is not None and isinstance(old_boxed, _UserValue):
                return
            box_type = StyleValue
            new_value = value.value
        else:
            box_type = _UserValue
            new_value = value

        new_value = self.convert(obj, name, new_value)
        new_boxed = box_type(new_value)        
        obj.__dict__[attr] = new_boxed
        if new_value != old_value:
            obj.trait_property_changed(name, old_value, new_value)

    def _default_value(self, obj, name):
        dv = self._default_value_kw
        if dv is not NO_VALUE:
            return dv
        
        factory = self._factory
        if factory is not None:
            return factory(obj, name)
        
        try:
            return self.create_default(obj, name)
        except NotImplementedError:
            pass

        msg = 'Cannot create a default value for %s' % name 
        raise ValueError(msg)
    
    def create_default(self, obj, name):
        raise NotImplementedError

    def convert(self, obj, name, value):
        raise NotImplementedError




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import os

from .qt.QtGui import QFileDialog
from .qt_dialog import QtDialog

from ...components.file_dialog import AbstractTkFileDialog
from ...guard import guard


ACCEPT_MODE_MAP = {
    'open': QFileDialog.AcceptOpen,
    'save': QFileDialog.AcceptSave,
}


class QtFileDialog(QtDialog, AbstractTkFileDialog):
    """ A Qt4 implementation of a FileDialog.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QFileDialog control.

        """
        self.widget = QFileDialog(parent)
    
    def initialize(self):
        """ Initializes the attributes of the underlying QFileDialog.

        """
        super(QtFileDialog, self).initialize()
        shell = self.shell_obj
        self.set_mode(shell.mode)
        self.set_multi_select(shell.multi_select)
        self.set_directory(shell.directory)
        self.set_filename(shell.filename)
        self.set_filters(shell.filters)
        self.set_selected_filter(shell.selected_filter)
        self.widget.setViewMode(QFileDialog.Detail)

    def bind(self):
        """ Binds the signal handlers for the file dialog.

        """
        super(QtFileDialog, self).bind()
        widget = self.widget
        widget.filesSelected.connect(self.on_files_selected)
        widget.filterSelected.connect(self.on_filter_selected)

    #--------------------------------------------------------------------------
    # Shell Object Change Handlers
    #--------------------------------------------------------------------------
    def shell_mode_changed(self, mode):
        """ Update the dialog for the given mode of behavior.

        """
        self.set_mode(mode)

    def shell_multi_select_changed(self, multi_select):
        """ Update the dialog for the given multi select behavior.

        """
        self.set_multi_select(multi_select)

    def shell_directory_changed(self, directory):
        """ Update the dialog with the new working directory.

        """
        if not guard.guarded(self, 'setting_directory'):
            self.set_directory(directory)
                
    def shell_filename_changed(self, filename):
        """ Update the dialog with the new selected filename.

        """
        if not guard.guarded(self, 'setting_filename'):
            self.set_filename(filename)

    def shell_filters_changed(self, filters):
        """ Update the dialog with the new filename filters.

        """
        # Traits doens't fire off a change event if an Enum updates
        # because its underlying values have changed. So, we just 
        # set the selected filter manually here to cover that case.
        self.set_filters(filters)
        self.set_selected_filter(self.shell_obj.selected_filter)

    def shell_selected_filter_changed(self, selected_filter):
        """ Update the dialog with the new selected filter.

        """
        if not guard.guarded(self, 'setting_selected_filter'):
            self.set_selected_filter(selected_filter)

    #--------------------------------------------------------------------------
    # Event Handlers 
    #--------------------------------------------------------------------------
    def on_files_selected(self, files):
        """ The signal handler for the dialog's `filesSelected` signal.

        """
        shell = self.shell_obj
        first_file = files[0] if files else u''
        with guard(self, 'setting_directory'):
            with guard(self, 'setting_filename'):
                shell.directory, shell.filename = os.path.split(first_file)
                shell._paths = files

    def on_filter_selected(self, qt_filter):
        """ The signal handler for the dialog's `filterSelected` signal.

        """
        shell = self.shell_obj
        with guard(self, 'setting_selected_filter'):
            shell.selected_filter = qt_filter

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_mode(self, mode):
        """ Sets the mode behavior of the file dialog.

        """
        self.widget.setAcceptMode(ACCEPT_MODE_MAP[mode])
    
    def set_multi_select(self, multi_select):
        """ Sets the multi select behavior of the file dialog.

        """
        widget = self.widget
        if widget.acceptMode() == QFileDialog.AcceptSave:
            mode = QFileDialog.AnyFile
        else:
            if multi_select:
                mode = QFileDialog.ExistingFile
            else:
                mode = QFileDialog.ExistingFiles
        widget.setFileMode(mode)

    def set_directory(self, directory):
        """ Sets the current directory of the dialog.

        """
        self.widget.setDirectory(directory)
    
    def set_filename(self, filename):
        """ Sets the selected filename in the dialog.

        """
        self.widget.selectFile(filename)
    
    def set_filters(self, filters):
        """ Sets the name filters in the file dialog.

        """
        self.widget.setNameFilters(filters)
    
    def set_selected_filter(self, selected_filter):
        """ Sets the selected name filter in the file dialog.

        """
        self.widget.selectNameFilter(selected_filter)

    def set_central_widget(self, widget):
        """ Overridden parent class method which makes central widget
        operations a no-op, since a file dialog has no children.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from unittest import TestCase

from traits.api import TraitError

from ..components.constraints_widget import ConstraintsWidget


class TestLayoutComponent(TestCase):
    """ Test basic operations on LayoutComponents.

    """
    def test_hug_property(self):
        """ Test that the hug property works correctly.

        """
        good_values = ['ignore', 'weak', 'medium', 'strong', 'required']
        bad_values = ['Ignore', None, 'srong']

        comp = ConstraintsWidget()
        self.assertEqual(comp.hug_width, 'strong')
        self.assertEqual(comp.hug_height, 'strong')
        self.assertEqual(comp.hug, ('strong', 'strong'))
        comp.hug_width = 'ignore'
        self.assertEqual(comp.hug, ('ignore', 'strong'))
        comp.hug_height = 'medium'
        self.assertEqual(comp.hug, ('ignore', 'medium'))
        comp.hug = ('weak', 'required')

        for good_width in good_values:
            for good_height in good_values:
                comp.hug = (good_width, good_height)
                self.assertEqual(comp.hug_width, good_width)
                self.assertEqual(comp.hug_height, good_height)

        for good_width in good_values:
            for bad_height in bad_values:
                self.assertRaises(TraitError, comp.trait_set, hug=(good_width, bad_height))

        for bad_width in good_values:
            for good_height in bad_values:
                self.assertRaises(TraitError, comp.trait_set, hug=(bad_width, good_height))

    def test_resist_clip_property(self):
        """ Test that the resist_clip property works correctly.

        """
        good_values = ['ignore', 'weak', 'medium', 'strong', 'required']
        bad_values = ['Ignore', None, 'srong']

        comp = ConstraintsWidget()
        self.assertEqual(comp.resist_clip_width, 'strong')
        self.assertEqual(comp.resist_clip_height, 'strong')
        self.assertEqual(comp.resist_clip, ('strong', 'strong'))
        comp.resist_clip_width = 'ignore'
        self.assertEqual(comp.resist_clip, ('ignore', 'strong'))
        comp.resist_clip_height = 'medium'
        self.assertEqual(comp.resist_clip, ('ignore', 'medium'))
        comp.resist_clip = ('weak', 'required')

        for good_width in good_values:
            for good_height in good_values:
                comp.resist_clip = (good_width, good_height)
                self.assertEqual(comp.resist_clip_width, good_width)
                self.assertEqual(comp.resist_clip_height, good_height)

        for good_width in good_values:
            for bad_height in bad_values:
                self.assertRaises(TraitError, comp.trait_set, resist_clip=(good_width, bad_height))

        for bad_width in good_values:
            for good_height in bad_values:
                self.assertRaises(TraitError, comp.trait_set, resist_clip=(bad_width, good_height))




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Unicode, Instance

from .dialog import AbstractTkDialog, Dialog


class AbstractTkDirectoryDialog(AbstractTkDialog):
    """ The abstract toolkit interface for a DirectoryDialog.

    """
    @abstractmethod
    def shell_directory_changed(self, directory):
        """ Update the dialog with the new working directory.

        """
        raise NotImplementedError


class DirectoryDialog(Dialog):
    """ A dialog widget that allows the user to select directories.

    """
    #: The current directory of the dialog.
    directory = Unicode

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkDirectoryDialog)

    #--------------------------------------------------------------------------
    # Parent Class Overrides 
    #--------------------------------------------------------------------------
    def add_subcomponent(self, component):
        """ An overriden parent class method which prevents subcomponents
        from being declared for a DirectoryDialog instance.

        """
        msg = "Cannot add subcomponents to a DirectoryDialog."
        raise ValueError(msg)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant
from .. import date_edit

from enaml.backends.qt.qt_date_edit import qdate_to_python

class TestQtDateEdit(QtTestAssistant, date_edit.TestDateEdit):
    """ QtDateEdit tests. """

    def get_date(self, widget):
        """  Get the toolkits widget's active date.

        """
        date = widget.date()
        return qdate_to_python(date)

    def get_min_date(self, widget):
        """  Get the toolkits widget's maximum date attribute.

        """
        date = widget.minimumDate()
        return qdate_to_python(date)

    def get_max_date(self, widget):
        """ Get the toolkits widget's minimum date attribute.

        """
        date = widget.maximumDate()
        return qdate_to_python(date)

    def change_date(self, widget, date):
        """ Simulate a change date action at the toolkit widget.

        .. note:: The setDate method in Qt will also signal the dateChanged
            signal.
        """
        self.widget.setDate(date)

    def get_date_as_string(self, widget):
        """  Get the toolkits widget's active date as a string.

        """
        return self.widget.text()




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from ....noncomponents.abstract_image import AbstractTkImage


FLAG_MAP = {
    'bmp': wx.BITMAP_TYPE_BMP,
    'gif': wx.BITMAP_TYPE_GIF,
    'jpg': wx.BITMAP_TYPE_JPEG,
    'jpeg': wx.BITMAP_TYPE_JPEG,
    'png': wx.BITMAP_TYPE_PNG,
    'pcx': wx.BITMAP_TYPE_PCX,
    'pnm': wx.BITMAP_TYPE_PNM,
    'tif': wx.BITMAP_TYPE_TIF,
    'tiff': wx.BITMAP_TYPE_TIF,
    'tga': wx.BITMAP_TYPE_TGA,
    'xpm': wx.BITMAP_TYPE_XPM,
    'ico': wx.BITMAP_TYPE_ICO,
    'cur': wx.BITMAP_TYPE_CUR,
    'ani': wx.BITMAP_TYPE_ANI,
    '': wx.BITMAP_TYPE_ANY,
}


class WXImage(AbstractTkImage):
    """ A Wx implementation of AbstractTkImage.
    
    """
    def __init__(self, wximage=None):
        """ Initialize a WXImage.

        Parameters
        ----------
        wximage : wx.Image instance or None, optional
            A wx.Image instance which holds the data. If None is passed, 
            then the wx.NullImage will be used.

        """
        if wximage is None:
            wximage = wx.NullImage
        self._wximage = wximage
    
    #--------------------------------------------------------------------------
    # Abstract API Implementation
    #--------------------------------------------------------------------------
    @classmethod
    def from_data(cls, data, size):
        """ Construct an image from a bytearray of RGBA bytes.

        Parameters
        ----------
        data : bytearray
            A bytearray of the image data in RGBA32 format.

        size : (width, height)
            The width, height size tuple of the image.

        Returns
        -------
        results : AbstractTkImage
            An appropriate image instance.

        """
        w, h = size

        # Split the array into color and alpha to satisfy WX
        rgb_array = bytearray(w*h*3)
        alpha_array = bytearray(w*h)
        
        rgb_array[0::3] = data[0::4]
        rgb_array[1::3] = data[1::4]
        rgb_array[2::3] = data[2::4]
        alpha_array[:] = data[3::4]

        return cls(wx.ImageFromDataWithAlpha(w, h, rgb_array, alpha_array))

    @classmethod
    def from_file(cls, path, format=''):
        """ Read in the image data from a file.

        Parameters
        ----------
        path : string
            The path to the image file on disk.

        format : string, optional
            The image format of the data in the file. If not given,
            then the image format will be determined automatically
            if possible.
        
        Returns
        -------
        results : AbstractTkImage
            An appropriate image instance.

        """
        flag = FLAG_MAP.get(format.lower(), wx.BITMAP_TYPE_ANY)
        return cls(wx.Image(path, flag))

    @property
    def size(self):
        """ The size of the image as a (width, height) tuple.
        
        Returns
        -------
        result : (width, height)
            The width, height size tuple of the image.

        """
        img = self._wximage
        return (img.GetWidth(), img.GetHeight())

    def data(self):
        """ The data for the image as a bytearray of RGBA bytes.
        
        Returns
        -------
        results : bytearray
            A bytearray of the image data in RGBA32 format.

        """
        wximg = self._wximage
        width, height = wximg.GetWidth(), wximg.GetHeight()
        buf = wximg.GetDataBuffer()
        out = bytearray(width * height * 4)
        out[0::4] = buf[0::3]
        out[1::4] = buf[1::3]
        out[2::4] = buf[2::3]   
        if wximg.HasAlpha():
            out[3::4] = wximg.GetAlphaBuffer()[:]
        else:
            out[3::4] = '\xff' * (width * height)
        return out

    def scale(self, size):
        """ Create a new version of this image scaled to the given size.
        
        Parameters
        ----------
        size : (width, height)
            The size of the scaled image.
            
        Returns
        -------
        results : AbstractTkImage
            A new image of the proper scaled size.
            
        """
        width, height = size
        new_img = self._wximage.Scale(width, height, wx.IMAGE_QUALITY_HIGH)
        return WXImage(new_img)
        
    #--------------------------------------------------------------------------
    # Toolkit Specific
    #--------------------------------------------------------------------------
    def as_wxImage(self):
        """ Returns the internal wx.Image instance.

        """
        return self._wximage

    def as_wxBitmap(self):
        """ Returns a wx.Bitmap generated from the underlying wx.Image.
        
        This is provided as a convenience for wx backend components 
        which require a wx.Bitmap rather than a wx.Image.
        
        """
        return wx.BitmapFromImage(self._wximage)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import datetime
import locale
import sys
import unittest

from enaml.localization import SystemLocale


@unittest.skip('Skipping locale tests')
class TestSystemLocale(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        if sys.platform == 'win32':
            locale.setlocale(locale.LC_ALL, 'usa')
        else:
            locale.setlocale(locale.LC_ALL, ('en_US', 'UTF-8'))
    
    @classmethod
    def tearDownClass(cls):
        locale.setlocale(locale.LC_ALL, 'C')

    def setUp(self):
        self.locale = SystemLocale.default()

    def tearDown(self):
        del self.locale
    
    def test_name(self):
        self.assertEqual(self.locale.name, u'en_US')
    
    def test_encoding(self):
        self.assertEqual(self.locale.encoding, u'UTF-8')
    
    def test_am_string(self):
        self.assertEqual(self.locale.am_string, u'AM')
    
    def test_pm_string(self):
        self.assertEqual(self.locale.pm_string, u'PM')
    
    def test_decimal_point(self):
        self.assertEqual(self.locale.decimal_point, u'.')
    
    def test_monetary_decimal_point(self):
        self.assertEqual(self.locale.monetary_decimal_point, u'.')
    
    def test_exponential(self):
        self.assertEqual(self.locale.exponential, u'e')
    
    def test_thousands_separator(self):
        self.assertEqual(self.locale.thousands_separator, u',')
    
    def test_monetary_thousands_separator(self):
        self.assertEqual(self.locale.monetary_thousands_separator, u',')
    
    def test_negative_sign(self):
        self.assertEqual(self.locale.negative_sign, u'-')

    def test_positive_sign(self):
        self.assertEqual(self.locale.positive_sign, u'')
    
    def test_month_name(self):
        self.assertEqual(self.locale.month_name(1), u'January')
        self.assertEqual(self.locale.month_name(2), u'February')
        self.assertEqual(self.locale.month_name(3), u'March')
        self.assertEqual(self.locale.month_name(4), u'April')
        self.assertEqual(self.locale.month_name(5), u'May')
        self.assertEqual(self.locale.month_name(6), u'June')
        self.assertEqual(self.locale.month_name(7), u'July')
        self.assertEqual(self.locale.month_name(8), u'August')
        self.assertEqual(self.locale.month_name(9), u'September')
        self.assertEqual(self.locale.month_name(10), u'October')
        self.assertEqual(self.locale.month_name(11), u'November')
        self.assertEqual(self.locale.month_name(12), u'December')

    def test_month_name_abbr(self):
        self.assertEqual(self.locale.month_name(1, abbreviated=True), u'Jan')
        self.assertEqual(self.locale.month_name(2, abbreviated=True), u'Feb')
        self.assertEqual(self.locale.month_name(3, abbreviated=True), u'Mar')
        self.assertEqual(self.locale.month_name(4, abbreviated=True), u'Apr')
        self.assertEqual(self.locale.month_name(5, abbreviated=True), u'May')
        self.assertEqual(self.locale.month_name(6, abbreviated=True), u'Jun')
        self.assertEqual(self.locale.month_name(7, abbreviated=True), u'Jul')
        self.assertEqual(self.locale.month_name(8, abbreviated=True), u'Aug')
        self.assertEqual(self.locale.month_name(9, abbreviated=True), u'Sep')
        self.assertEqual(self.locale.month_name(10, abbreviated=True), u'Oct')
        self.assertEqual(self.locale.month_name(11, abbreviated=True), u'Nov')
        self.assertEqual(self.locale.month_name(12, abbreviated=True), u'Dec')

    def test_day_name(self):
        self.assertEqual(self.locale.day_name(1), u'Sunday')
        self.assertEqual(self.locale.day_name(2), u'Monday')
        self.assertEqual(self.locale.day_name(3), u'Tuesday')
        self.assertEqual(self.locale.day_name(4), u'Wednesday')
        self.assertEqual(self.locale.day_name(5), u'Thursday')
        self.assertEqual(self.locale.day_name(6), u'Friday')
        self.assertEqual(self.locale.day_name(7), u'Saturday')

    def test_day_name_abbr(self):
        self.assertEqual(self.locale.day_name(1, abbreviated=True), u'Sun')
        self.assertEqual(self.locale.day_name(2, abbreviated=True), u'Mon')
        self.assertEqual(self.locale.day_name(3, abbreviated=True), u'Tue')
        self.assertEqual(self.locale.day_name(4, abbreviated=True), u'Wed')
        self.assertEqual(self.locale.day_name(5, abbreviated=True), u'Thu')
        self.assertEqual(self.locale.day_name(6, abbreviated=True), u'Fri')
        self.assertEqual(self.locale.day_name(7, abbreviated=True), u'Sat')

    def test_time_format(self):
        self.assertEqual(self.locale.time_format(), u'%I:%M:%S %p')
        self.assertEqual(self.locale.time_format(abbreviated=True), u'%H:%M:%S')

    def test_date_format(self):
        self.assertEqual(self.locale.date_format(), u'%m/%d/%Y')
        self.assertEqual(self.locale.date_format(abbreviated=True), u'%m/%d/%Y')

    def test_datetime_format(self):
        self.assertEqual(self.locale.datetime_format(), u'%a %b %d %X %Y')
        self.assertEqual(self.locale.datetime_format(abbreviated=True), u'%a %b %d %X %Y')

    def test_to_time(self):
        t1 = datetime.time(15, 27, 38)
        t2 = datetime.time(4, 19, 42)
        self.assertEqual(self.locale.to_time('3:27:38 PM'), t1)
        self.assertEqual(self.locale.to_time('15:27:38', abbreviated=True), t1)
        self.assertEqual(self.locale.to_time('4:19:42 AM'), t2)
        self.assertEqual(self.locale.to_time('4:19:42', abbreviated=True), t2)

    def test_from_time(self):
        t1 = datetime.time(15, 27, 38)
        t2 = datetime.time(4, 19, 42)
        self.assertEqual(self.locale.from_time(t1), u'03:27:38 PM')
        self.assertEqual(self.locale.from_time(t1, abbreviated=True), u'15:27:38')
        self.assertEqual(self.locale.from_time(t2), u'04:19:42 AM')
        self.assertEqual(self.locale.from_time(t2, abbreviated=True), u'04:19:42')

    def test_to_date(self):
        d1 = datetime.date(1943, 3, 28)
        d2 = datetime.date(2050, 12, 20)
        self.assertEqual(self.locale.to_date('3/28/1943'), d1)
        self.assertEqual(self.locale.to_date('3/28/1943', abbreviated=True), d1)
        self.assertEqual(self.locale.to_date('03/28/1943'), d1)
        self.assertEqual(self.locale.to_date('03/28/1943', abbreviated=True), d1)
        self.assertEqual(self.locale.to_date('12/20/2050'), d2)
        self.assertEqual(self.locale.to_date('12/20/2050', abbreviated=True), d2)

    def test_from_date(self):
        d1 = datetime.date(1943, 3, 28)
        d2 = datetime.date(2050, 12, 20)
        self.assertEqual(self.locale.from_date(d1), u'03/28/1943')
        self.assertEqual(self.locale.from_date(d1, abbreviated=True), u'03/28/1943')
        self.assertEqual(self.locale.from_date(d2), u'12/20/2050')
        self.assertEqual(self.locale.from_date(d2, abbreviated=True), u'12/20/2050')

    def test_to_datetime(self):
        d1 = datetime.datetime(1943, 3, 28, 15, 27, 38)
        d2 = datetime.datetime(2050, 12, 20, 4, 19, 42)
        self.assertEqual(self.locale.to_datetime('Sun Mar 28 15:27:38 1943'), d1)
        self.assertEqual(self.locale.to_datetime('Sun Mar 28 15:27:38 1943', abbreviated=True), d1)
        self.assertEqual(self.locale.to_datetime('Tue Dec 20 4:19:42 2050'), d2)
        self.assertEqual(self.locale.to_datetime('Tue Dec 20 4:19:42 2050', abbreviated=True), d2)

    def test_from_datetime(self):
        d1 = datetime.datetime(1943, 3, 28, 15, 27, 38)
        d2 = datetime.datetime(2050, 12, 20, 4, 19, 42)
        self.assertEqual(self.locale.from_datetime(d1), u'Sun Mar 28 15:27:38 1943')
        self.assertEqual(self.locale.from_datetime(d1, abbreviated=True), u'Sun Mar 28 15:27:38 1943')
        self.assertEqual(self.locale.from_datetime(d2), u'Tue Dec 20 04:19:42 2050')
        self.assertEqual(self.locale.from_datetime(d2, abbreviated=True), u'Tue Dec 20 04:19:42 2050')

    def test_to_int(self):
        self.assertEqual(self.locale.to_int('12'), 12)
        self.assertEqual(self.locale.to_int('12,345,678'), 12345678)
        self.assertEqual(self.locale.to_int('10', base=2), 2)
        self.assertEqual(self.locale.to_int('0B10', base=2), 2)
        self.assertEqual(self.locale.to_int('0b10', base=2), 2)
        self.assertEqual(self.locale.to_int('17', base=8), 15)
        self.assertEqual(self.locale.to_int('017', base=8), 15)
        self.assertEqual(self.locale.to_int('FF', base=16), 255)
        self.assertEqual(self.locale.to_int('ff', base=16), 255)
        self.assertEqual(self.locale.to_int('0XFF', base=16), 255)
        self.assertEqual(self.locale.to_int('0xFF', base=16), 255)
        self.assertEqual(self.locale.to_int('0Xff', base=16), 255)
        self.assertEqual(self.locale.to_int('0xff', base=16), 255)
        self.assertEqual(self.locale.to_int('456', base=17), 1247)
        self.assertEqual(self.locale.to_int('456', base=36), 5370)
        self.assertEqual(self.locale.to_int('456', base=7), 237)
        self.assertEqual(self.locale.to_int('12AZ', base=36), 49643)

    def test_from_int(self):
        self.assertEqual(self.locale.from_int(12), u'12')
        self.assertEqual(self.locale.from_int(12345678, group=True), u'12,345,678')
        self.assertEqual(self.locale.from_int(2, base=2), u'0b10')
        self.assertEqual(self.locale.from_int(15, base=8), u'017')
        self.assertEqual(self.locale.from_int(255, base=16), u'0XFF')
        self.assertEqual(self.locale.from_int(1247, base=17), '456')
        self.assertEqual(self.locale.from_int(5370, base=36), '456')
        self.assertEqual(self.locale.from_int(237, base=7), '456')
        self.assertEqual(self.locale.from_int(49643, base=36), '12AZ')

    def test_to_long(self):
        self.assertEqual(self.locale.to_long('12345678901'), 12345678901L)
        self.assertEqual(self.locale.to_long('12,345,678,901'), 12345678901L)
        self.assertEqual(self.locale.to_long('1'*32, base=2), 4294967295L)
        self.assertEqual(self.locale.to_long('0B' + '1'*32, base=2), 4294967295L)
        self.assertEqual(self.locale.to_long('0b' + '1'*32, base=2), 4294967295L)
        self.assertEqual(self.locale.to_long('1'*12, base=8), 9817068105L)
        self.assertEqual(self.locale.to_long('0' + '1'*12, base=8), 9817068105L)
        self.assertEqual(self.locale.to_long('F'*8, base=16), 4294967295L)
        self.assertEqual(self.locale.to_long('f'*8, base=16), 4294967295L)
        self.assertEqual(self.locale.to_long('0X' + 'F'*8, base=16), 4294967295L)
        self.assertEqual(self.locale.to_long('0X' + 'f'*8, base=16), 4294967295L)
        self.assertEqual(self.locale.to_long('0x' + 'F'*8, base=16), 4294967295L)
        self.assertEqual(self.locale.to_long('0x' + 'f'*8, base=16), 4294967295L)
        self.assertEqual(self.locale.to_long('456'*3, base=17), 30105676301L)
        self.assertEqual(self.locale.to_long('456'*3, base=36), 11689571692410L)
        self.assertEqual(self.locale.to_long('456'*4, base=7), 9591769200L)

    def test_from_long(self):
        self.assertEqual(self.locale.from_long(12L), u'12')
        self.assertEqual(self.locale.from_long(12345678L, group=True), u'12,345,678')
        self.assertEqual(self.locale.from_long(2L, base=2), u'0b10')
        self.assertEqual(self.locale.from_long(15L, base=8), u'017')
        self.assertEqual(self.locale.from_long(255L, base=16), u'0XFF')
        self.assertEqual(self.locale.from_long(1247L, base=17), u'456')
        self.assertEqual(self.locale.from_long(5370L, base=36), u'456')
        self.assertEqual(self.locale.from_long(237L, base=7), u'456')
        self.assertEqual(self.locale.from_long(49643L, base=36), u'12AZ')

    def test_to_float(self):
        self.assertAlmostEqual(self.locale.to_float('12'), 12.0)
        self.assertAlmostEqual(self.locale.to_float('12.42'), 12.42)
        self.assertAlmostEqual(self.locale.to_float('12,345,644.23'), 12345644.23)
        self.assertAlmostEqual(self.locale.to_float('12345644.23'), 12345644.23)
        self.assertAlmostEqual(self.locale.to_float('0.0000123'), 0.0000123)
        self.assertAlmostEqual(self.locale.to_float('123.45678'), 123.45678)
        self.assertAlmostEqual(self.locale.to_float('123e-4'), 123e-4)
        self.assertAlmostEqual(self.locale.to_float('123E-4'), 123e-4)
    
    def test_from_float(self):
        self.assertEqual(self.locale.from_float(12.0), u'12')
        self.assertEqual(self.locale.from_float(12.42), u'12.42')
        self.assertEqual(self.locale.from_float(12345644.23), u'1.23456e+07')
        self.assertEqual(self.locale.from_float(12345644.23, prec=10), u'12345644.23')
        self.assertEqual(self.locale.from_float(12345644.23, group=True, prec=10), u'12,345,644.23')
        self.assertEqual(self.locale.from_float(0.0000123), u'1.23e-05')
        self.assertEqual(self.locale.from_float(0.00123), u'0.00123')
        self.assertEqual(self.locale.from_float(123.456), u'123.456')
        self.assertEqual(self.locale.from_float(123e-4), u'0.0123')

    def test_to_complex(self):
        self.assertAlmostEqual(self.locale.to_complex('12'), complex(12))
        self.assertAlmostEqual(self.locale.to_complex('12.5'), complex(12.5))
        self.assertAlmostEqual(self.locale.to_complex('12.5j'), 12.5j)
        self.assertAlmostEqual(self.locale.to_complex('12.3+12.4j'), 12.3+12.4j)
        self.assertAlmostEqual(self.locale.to_complex('11.4-6.32j'), 11.4-6.32j)
        self.assertAlmostEqual(self.locale.to_complex('-3-5j'), -3-5j)
    
    def test_from_complex(self):
        self.assertEqual(self.locale.from_complex(12), u'12+0j')
        self.assertEqual(self.locale.from_complex(complex(12.5)), u'12.5+0j')
        self.assertEqual(self.locale.from_complex(12.5j), u'0+12.5j')
        self.assertEqual(self.locale.from_complex(12.3+12.4j), u'12.3+12.4j')
        self.assertEqual(self.locale.from_complex(11.4-6.32j), u'11.4-6.32j')
        self.assertEqual(self.locale.from_complex(-3-5j), u'-3-5j')

    def test_to_currency(self):
        self.assertAlmostEqual(self.locale.to_currency('$4.56'), 4.56)
        self.assertAlmostEqual(self.locale.to_currency('USD 4.56', intl=True), 4.56)
        self.assertAlmostEqual(self.locale.to_currency('$123456.45'), 123456.45) 
        self.assertAlmostEqual(self.locale.to_currency('$123,456.45', group=True), 123456.45)
        self.assertAlmostEqual(self.locale.to_currency('123,456.45', symbol=False, group=True), 123456.45)

    def test_from_currency(self):
        self.assertEqual(self.locale.from_currency(4.56), u'$4.56')
        self.assertEqual(self.locale.from_currency(4.56, intl=True), u'USD 4.56')
        self.assertEqual(self.locale.from_currency(123456.45), u'$123456.45') 
        self.assertEqual(self.locale.from_currency(123456.45, group=True), u'$123,456.45')
        self.assertEqual(self.locale.from_currency(123456.45, symbol=False, group=True), u'123,456.45')        




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .abstract_item_model import (
    AbstractListModel, AbstractTableModel, ITEM_IS_SELECTABLE,
    ITEM_IS_ENABLED, ITEM_IS_EDITABLE,
)


#------------------------------------------------------------------------------
# List Model
#------------------------------------------------------------------------------
class ListModel(AbstractListModel):
    """ A concrete implementation of AbstractListModel which is intended
    to be easy to use for data models which behave more-or-less like
    one dimensional sequences.

    The data object can be updated dynamically after instantiation by 
    using the 'data_source' property.

    """
    base_flags = ITEM_IS_ENABLED | ITEM_IS_SELECTABLE

    def __init__(self, data, editable=False, display_data_converter=unicode,
                 edit_data_converter=None, background_brush_func=None, 
                 foreground_brush_func=None, font_func=None, 
                 vertical_headers=None, horizontal_headers=None):
        """ Initialize a ListModel.

        Parameters
        ----------
        data : 1D sequence-like object
            A sequence-like object that provides the data for the model.
            At a minimum it must support __getitem__ and __len__, and
            if editable, __setitem__.
        
        editable : bool, optional
            A bool which indicates whether or not the model is editable.
            If True, editing a cell will be done with an editor that is
            appropriate for the type of data given in the model. The
            default is False.
            
        display_data_converter : callable, optional
            An optional callable which should take a single argument: the
            value for the cell, and return a unicode value to use as the
            display value in the cell. The default value is the builtin
            unicode object.
        
        edit_data_converter : callable or None, optional
            An optional callable which should take a single argument: the
            unicode value typed by the user, and return a value to set
            in the model. This converter is only used when the model is
            declared as editable. If this converter is not supplied and
            the model is editable, then the type of the converted value
            will be determined automatically based on the input type. If
            a converter is supplied, and it cannot convert the value, it
            should raise a ValueError and the change will be ignored. The
            default value is None.
        
        background_brush_func : callable or None, optional
            If provided, it should be a callable which accepts two 
            arguments: the data value and the row index, and returns
            a brush for the background of the cell. The default is
            None.
        
        foreground_brush_func : callable or None, optional
            If provided, it should be a callable which accepts two 
            arguments: the data value and the row index, and returns
            a brush for the foreground of the cell. The default is
            None.
        
        font_func : callable or None, optional
            If provided, it should be a callable which accepts two 
            arguments: the data value and the row index, and returns
            a font for the cell. The default is None.
        
        vertical_headers : sequence-like object or None
            If provided, is should be a sequence like object which
            will be indexed with an integer index to retrieve a
            unicode string for the given row header. 
        
        horizontal_headers : sequence-like object or None
            If provided, is should be a sequence like object which
            will be indexed with an integer index to retrieve a
            unicode string for the given column header.

        """
        self._data_source = data
        self._editable = editable
        self._display_data_converter = display_data_converter
        self._edit_data_converter = edit_data_converter
        self._background_brush_func = background_brush_func
        self._foreground_brush_func = foreground_brush_func
        self._font_func = font_func
        self._vertical_headers = vertical_headers
        self._horizontal_headers = horizontal_headers

    def _get_data_source(self):
        """ The property getter for the 'data_source' property.
        
        """
        return self._data_source
    
    def _set_data_source(self, data):
        """ The property setter for the 'data_source' property.    
        
        """
        self.begin_reset_model()
        self._data_source = data
        self.end_reset_model()
    
    data_source = property(_get_data_source, _set_data_source)

    def flags(self, index):
        """ Returns the flags for the items in the model.

        """
        flags = self.base_flags
        if self._editable:
            flags |= ITEM_IS_EDITABLE
        return flags

    def data(self, index):
        """ Returns the data point from the data source converted to a 
        unicode for display.

        """
        return self._display_data_converter(self._data_source[index.row])
    
    def edit_data(self, index):
        """ Returns the data value for editing. If an edit converter is
        provided, then the unicode form of the data is returned, 
        otherwise, the raw datapoint is returned.

        """
        if self._edit_data_converter is None:
            return self._data_source[index.row]
        return self.data(index)
    
    def set_data(self, index, value):
        """ Sets the data source with the converted value, emits the 
        proper changed notification and returns True. 

        """
        converter = self._edit_data_converter
        if converter is not None:
            try:
                value = converter(value)
            except ValueError:
                return False
        self._data_source[index.row] = value
        self.notify_data_changed(index, index)
        return True

    def row_count(self, parent=None):
        """ Returns the number of rows in the data source.

        """
        if parent is not None:
            return 0
        return len(self._data_source)

    def background(self, index):
        """ Returns the brush for the background of the given index,
        or None.

        """
        brush_func = self._background_brush_func
        if brush_func is not None:
            row = index.row
            return brush_func(self._data_source[row], row)
    
    def foreground(self, index):
        """ Returns the brush for the foreground of the given index,
        or None.

        """
        brush_func = self._foreground_brush_func
        if brush_func is not None:
            row = index.row
            return brush_func(self._data_source[row], row)

    def font(self, index):
        """ Returns the font for the foreground of the given index,
        or None.

        """
        font_func = self._font_func
        if font_func is not None:
            row = index.row
            return font_func(self._data_source[row], row)

    def vertical_header_data(self, section):
        """ Returns the vertical header data for the given section.

        """
        headers = self._vertical_headers
        if headers is not None:
            res = headers[section]
        else:
            sup = super(ListModel, self)
            res = sup.vertical_header_data(section)
        return res
    
    def horizontal_header_data(self, section):
        """ Returns the horiztonal header data for the given section.

        """
        headers = self._horizontal_headers
        if headers is not None:
            res = headers[section]
        else:
            sup = super(ListModel, self)
            res = sup.horizontal_header_data(section)
        return res


#------------------------------------------------------------------------------
# Table Model
#------------------------------------------------------------------------------
class TableModel(AbstractTableModel):
    """ A concrete implementation of AbstractTableModel which is intended
    to be easy to use for data models which behave like two dimensional
    arrays.

    The data object can be updated dynamically after instantiation by 
    using the 'data_source' property.

    """
    base_flags = ITEM_IS_ENABLED | ITEM_IS_SELECTABLE

    def __init__(self, data, editable=False, display_data_converter=unicode,
                 edit_data_converter=None, background_brush_func=None, 
                 foreground_brush_func=None, font_func=None,
                 vertical_headers=None, horizontal_headers=None):
        """ Initialize a TableModel.

        Parameters
        ----------
        data : 2D array-like object
            An array-like object that provides the data for the model.
            At a minimum it must support __getitem__ which can be
            indexed with a (row, col) tuple, and have a 'shape' attribute
            which gives an (nrows, ncols) tuple. If the model is also
            editable, it should support __setitem__

        editable : bool, optional
            A bool which indicates whether or not the model is editable.
            If True, editing a cell will be done with an editor that is
            appropriate for the type of data given in the model. The
            default is False.
            
        display_converter : callable, optional
            An optional callable which should take a single argument: the
            value for the cell, and return a unicode value to use as the
            display value in the cell. The default value is the builtin
            unicode object.
        
        edit_converter : callable or None, optional
            An optional callable which should take a single argument: the
            unicode value typed by the user, and return a value to set
            in the model. This converter is only used when the model is
            declared as editable. If this converter is not supplied and
            the model is editable, then the type of the converted value
            will be determined automatically based on the input type. If
            a converter is supplied, and it cannot convert the value, it
            should raise a ValueError and the change will be ignored. The
            default value is None.
        
        background_brush_func : callable or None, optional
            If provided, it should be a callable which accepts three
            arguments: the data value, the row index and column index, 
            and returns a brush for the background of the cell. The 
            default is None.
        
        foreground_brush_func : callable or None, optional
            If provided, it should be a callable which accepts three
            arguments: the data value, the row index and column index, 
            and returns a brush for the foreground of the cell. The 
            default is None.
        
        font_func : callable or None, optional
            If provided, it should be a callable which accepts three
            arguments: the data value, the row index and column index, 
            and returns a font for the cell. The default is None.

        vertical_headers : sequence-like object or None
            If provided, is should be a sequence like object which
            will be indexed with an integer index to retrieve a
            unicode string for the given row header. 
        
        horizontal_headers : sequence-like object or None
            If provided, is should be a sequence like object which
            will be indexed with an integer index to retrieve a
            unicode string for the given column header.

        """
        self._data_source = data
        self._editable = editable
        self._display_data_converter = display_data_converter
        self._edit_data_converter = edit_data_converter
        self._background_brush_func = background_brush_func
        self._foreground_brush_func = foreground_brush_func
        self._font_func = font_func
        self._vertical_headers = vertical_headers
        self._horizontal_headers = horizontal_headers
    
    def _get_data_source(self):
        """ The property getter for the 'data_source' property.
        
        """
        return self._data_source
    
    def _set_data_source(self, data):
        """ The property setter for the 'data_source' property.    
        
        """
        self.begin_reset_model()
        self._data_source = data
        self.end_reset_model()
    
    data_source = property(_get_data_source, _set_data_source)

    def flags(self, index):
        """ Returns the flags for the items in the model.

        """
        flags = self.base_flags
        if self._editable:
            flags |= ITEM_IS_EDITABLE
        return flags

    def data(self, index): 
        """ Returns the data point from the data source converted to a 
        unicode for display.

        """
        row = index.row
        col = index.column
        return self._display_data_converter(self._data_source[row, col])
    
    def edit_data(self, index):
        """ Returns the data value for editing. If an edit converter is
        provided, then the unicode form of the data is returned, 
        otherwise, the raw datapoint is returned.

        """
        if self._edit_data_converter is None:
            row = index.row
            col = index.column
            return self._data_source[row, col]
        return self.data(index)
    
    def set_data(self, index, value):
        """ Sets the data source with the converted value, emits the 
        proper changed notification and returns True. 

        """
        converter = self._edit_data_converter
        if converter is not None:
            try:
                value = converter(value)
            except ValueError:
                return False
        row = index.row
        col = index.column
        self._data_source[row, col] = value
        self.notify_data_changed(index, index)
        return True
        
    def row_count(self, parent=None):
        """ Returns the number of rows in the data source.

        """
        if parent is not None:
            return 0
        return self._data_source.shape[0]
    
    def column_count(self, parent=None):
        """ Returns the number of columns in the data source.

        """
        if parent is not None:
            return 0
        return self._data_source.shape[1]

    def background(self, index):
        """ Returns the brush for the background of the given index,
        or None.

        """
        brush_func = self._background_brush_func
        if brush_func is not None:
            row = index.row
            col = index.column
            return brush_func(self._data_source[row, col], row, col)
    
    def foreground(self, index):
        """ Returns the brush for the foreground of the given index,
        or None.

        """
        brush_func = self._foreground_brush_func
        if brush_func is not None:
            row = index.row
            col = index.column
            return brush_func(self._data_source[row, col], row, col)

    def font(self, index):
        """ Returns the font for the foreground of the given index,
        or None.

        """
        font_func = self._font_func
        if font_func is not None:
            row = index.row
            col = index.column
            return font_func(self._data_source[row, col], row, col)

    def vertical_header_data(self, section):
        """ Returns the vertical header data for the given section.

        """
        headers = self._vertical_headers
        if headers is not None:
            res = headers[section]
        else:
            sup = super(TableModel, self)
            res = sup.vertical_header_data(section)
        return res
    
    def horizontal_header_data(self, section):
        """ Returns the horiztonal header data for the given section.

        """
        headers = self._horizontal_headers
        if headers is not None:
            res = headers[section]
        else:
            sup = super(TableModel, self)
            res = sup.horizontal_header_data(section)
        return res


#------------------------------------------------------------------------------
# Object Model
#------------------------------------------------------------------------------
class ObjectModel(AbstractTableModel):
    """ A concrete implementation of AbstractTableModel which is intended
    to be easy to use for data models which display list of objects 
    where each object corresponds to a row in the table.

    The data object can be updated dynamically after instantiation by 
    using the 'data_source' property.

    """
    base_flags = ITEM_IS_ENABLED | ITEM_IS_SELECTABLE

    @staticmethod
    def format_header(header):
        """ A simple formatter class which converts an attribute name
        in a form more-or-less suitable for a column header.

        """
        return ' '.join(s.capitalize() for s in header.split('_'))

    def __init__(self, data, transpose=False, editable=False, 
                 display_data_converter=unicode, edit_data_converter=None, 
                 background_brush_func=None, foreground_brush_func=None, 
                 font_func=None, vertical_headers=None, 
                 horizontal_headers=None):
        """ Initialize an ObjectModel.

        Parameters
        ----------
        data : a 2-tuple of (object, column_attributes)
            A tuple whose first element is a sequence-like object that
            provides the row objects for the model, and whose second
            element is a sequence-like object that provides the string 
            names of the attributes of the objects which provide the
            data for the columns. At a minimum both elements must support
            __getitem__ and __len__.

        transpose : bool, optional
            A boolean which indicates that the model should be transposed
            such that the objects represent columns and the attributes
            represent rows. The default is False.

        editable : bool, optional
            A bool which indicates whether or not the model is editable.
            If True, editing a cell will be done with an editor that is
            appropriate for the type of data given in the model. The
            default is False.
            
        display_converter : callable, optional
            An optional callable which should take a single argument: the
            value for the cell, and return a unicode value to use as the
            display value in the cell. The default value is the builtin
            unicode object.
        
        edit_converter : callable or None, optional
            An optional callable which should take a single argument: the
            unicode value typed by the user, and return a value to set
            in the model. This converter is only used when the model is
            declared as editable. If this converter is not supplied and
            the model is editable, then the type of the converted value
            will be determined automatically based on the input type. If
            a converter is supplied, and it cannot convert the value, it
            should raise a ValueError and the change will be ignored. The
            default value is None.
        
        background_brush_func : callable or None, optional
            If provided, it should be a callable which accepts three
            arguments: the data value, the row index and column index, 
            and returns a brush for the background of the cell. The 
            default is None.
        
        foreground_brush_func : callable or None, optional
            If provided, it should be a callable which accepts three
            arguments: the data value, the row index and column index, 
            and returns a brush for the foreground of the cell. The 
            default is None.
        
        font_func : callable or None, optional
            If provided, it should be a callable which accepts three
            arguments: the data value, the row index and column index, 
            and returns a font for the cell. The default is None.

        vertical_headers : sequence-like object or None
            If provided, is should be a sequence like object which
            will be indexed with an integer index to retrieve a
            unicode string for the given row header. 
        
        horizontal_headers : sequence-like object or None
            If provided, is should be a sequence like object which
            will be indexed with an integer index to retrieve a
            unicode string for the given column header.

        """
        self._data_source = data
        self._transpose = transpose
        self._editable = editable
        self._display_data_converter = display_data_converter
        self._edit_data_converter = edit_data_converter
        self._background_brush_func = background_brush_func
        self._foreground_brush_func = foreground_brush_func
        self._font_func = font_func
        self._vertical_headers = vertical_headers
        self._horizontal_headers = horizontal_headers
    
    def _get_data_source(self):
        """ The property getter for the 'data_source' property.
        
        """
        return self._data_source
    
    def _set_data_source(self, data):
        """ The property setter for the 'data_source' property.    
        
        """
        self.begin_reset_model()
        self._data_source = data
        self.end_reset_model()
    
    data_source = property(_get_data_source, _set_data_source)

    def _get_transpose(self):
        """ The property getter for the 'transpose' property.

        """
        return self._transpose
    
    def _set_transpose(self, value):
        """ The property setter for the 'transposed' property.

        """
        self.begin_reset_model()
        self._transpose = value
        self.end_reset_model()
    
    transpose = property(_get_transpose, _set_transpose)
    
    def flags(self, index):
        """ Returns the flags for the items in the model.

        """
        flags = self.base_flags
        if self._editable:
            flags |= ITEM_IS_EDITABLE
        return flags

    def data(self, index): 
        """ Returns the data point from the data source converted to a 
        unicode for display.

        """
        row = index.row
        col = index.column
        if self._transpose:
            row, col = col, row
        objs, attrs = self._data_source
        value = getattr(objs[row], attrs[col])
        return self._display_data_converter(value)
    
    def edit_data(self, index):
        """ Returns the data value for editing. If an edit converter is
        provided, then the unicode form of the data is returned, 
        otherwise, the raw datapoint is returned.

        """
        if self._edit_data_converter is None:
            row = index.row
            col = index.column
            if self._transpose:
                row, col = col, row
            objs, attrs = self._data_source
            return getattr(objs[row], attrs[col])
        return self.data(index)
    
    def set_data(self, index, value):
        """ Sets the data source with the converted value, emits the 
        proper changed notification and returns True. 

        """
        converter = self._edit_data_converter
        if converter is not None:
            try:
                value = converter(value)
            except ValueError:
                return False
        row = index.row
        col = index.column
        if self._transpose:
            row, col = col, row
        objs, attrs = self._data_source
        setattr(objs[row], attrs[col], value)
        self.notify_data_changed(index, index)
        return True
        
    def row_count(self, parent=None):
        """ Returns the number of rows in the data source.

        """
        if parent is not None:
            return 0
        idx = 0
        if self._transpose:
            idx = 1
        return len(self._data_source[idx])
    
    def column_count(self, parent=None):
        """ Returns the number of columns in the data source.

        """
        if parent is not None:
            return 0
        idx = 1
        if self._transpose:
            idx = 0
        return len(self._data_source[idx])

    def background(self, index):
        """ Returns the brush for the background of the given index,
        or None.

        """
        brush_func = self._background_brush_func
        if brush_func is not None:
            row = index.row
            col = index.column
            if self._transpose:
                row, col = col, row
            objs, attrs = self._data_source
            value = getattr(objs[row], attrs[col])
            return brush_func(value, row, col)
    
    def foreground(self, index):
        """ Returns the brush for the foreground of the given index,
        or None.

        """
        brush_func = self._foreground_brush_func
        if brush_func is not None:
            row = index.row
            col = index.column
            if self._transpose:
                row, col = col, row
            objs, attrs = self._data_source
            value = getattr(objs[row], attrs[col])
            return brush_func(value, row, col)

    def font(self, index):
        """ Returns the font for the foreground of the given index,
        or None.

        """
        font_func = self._font_func
        if font_func is not None:
            row = index.row
            col = index.column
            if self._transpose:
                row, col = col, row
            objs, attrs = self._data_source
            value = getattr(objs[row], attrs[col])
            return font_func(value, row, col)

    def vertical_header_data(self, section):
        """ Returns the vertical header data for the given section.

        """
        headers = self._vertical_headers
        if headers is not None:
            res = headers[section]
        else:
            if self._transpose:
                res = self.format_header(self._data_source[1][section])
            else:
                sup = super(ObjectModel, self)
                res = sup.vertical_header_data(section)
        return res
    
    def horizontal_header_data(self, section):
        """ Returns the horiztonal header data for the given section.

        """
        headers = self._horizontal_headers
        if headers is not None:
            res = headers[section]
        else:
            if not self._transpose:
                res = self.format_header(self._data_source[1][section])
            else:
                sup = super(ObjectModel, self)
                res = sup.horizontal_header_data(section)
        return res




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .coercing_validator import CoercingValidator


class BaseDTValidator(CoercingValidator):
    """ A CoercingValidator subclass which provides common functionality
    for the time, date, and datetime validators.

    """
    def __init__(self, abbreviated=True, **kwargs):
        """ Initialize a BaseDTValidator.

        Parameters
        ----------
        abbreviated : bool, optional
            Whether or not to use abbreviated time formatting. Defaults
            to True.
        
        **kwargs
            The keyword arguments necessary to initialize an instance
            of CoercingValidator.

        """
        super(BaseDTValidator, self).__init__(**kwargs)
        self.abbreviated = abbreviated

    def validate(self, text):
        """ Validates the input text against the rules of the validator.

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.
        
        Returns
        -------
        result : int
            One of the class attribute constants INVALID, INTERMEDIATE, 
            or VALID.
        
        Notes
        -----
        The string is INVALID under the following conditions:
            - The string fails coercing validation.
        
        The string is INTERMEDIATE under the following conditions:
            - The string is empty.

        The string is VALID for all other situations.

        """
        if not text:
            return self.INTERMEDIATE
        return self._validate(text)[0]


class TimeValidator(BaseDTValidator):
    """ A BaseDTValidator subclass which uses the locale information to 
    convert to and from datetime.time objects.

    """
    def _convert(self, text):
        """ Overridden default convert method which uses the locale 
        object to convert the text into a datetime.time object.

        """
        return self.locale.to_time(text, abbreviated=self.abbreviated)

    def _format(self, value):
        """ Overridden default convert method which uses the locale 
        object to format the datetime.time object into a string.

        """
        return self.locale.from_time(value, abbreviated=self.abbreviated)


class DateValidator(BaseDTValidator):
    """ A BaseDTValidator subclass which uses the locale information to 
    convert to and from datetime.date objects.

    """
    def _convert(self, text):
        """ Overridden default convert method which uses the locale 
        object to convert the text into a datetime.date object.

        """
        return self.locale.to_date(text, abbreviated=self.abbreviated)

    def _format(self, value):
        """ Overridden default convert method which uses the locale 
        object to format the datetime.time object into a string.

        """
        return self.locale.from_date(value, abbreviated=self.abbreviated)


class DatetimeValidator(BaseDTValidator):
    """ A BaseDTValidator subclass which uses the locale information to 
    convert to and from datetime.datetime objects.

    """
    def _convert(self, text):
        """ Overridden default convert method which uses the locale 
        object to convert the text into a datetime.datetime object.

        """
        return self.locale.to_datetime(text, abbreviated=self.abbreviated)

    def _format(self, value):
        """ Overridden default convert method which uses the locale 
        object to format the datetime.time object into a string.

        """
        return self.locale.from_datetime(value, abbreviated=self.abbreviated)




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import os
import re

import wx

from .wx_dialog import WXDialog

from ...components.file_dialog import AbstractTkFileDialog
from ...guard import guard


STYLE_MODE_MAP = {
    'open': wx.FD_OPEN,
    'save': wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
}


FILTER_RE = re.compile(r'\((.*?\*.*?)\)')


class WXFileDialog(WXDialog, AbstractTkFileDialog):
    """ A Wx implementation of a FileDialog.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.FileDialog control.

        """
        self.widget = wx.FileDialog(parent)

    def initialize(self):
        """ Initializes the attributes of the underlying QFileDialog.

        """
        super(WXFileDialog, self).initialize()
        shell = self.shell_obj
        self.set_mode(shell.mode)
        self.set_multi_select(shell.multi_select)
        self.set_directory(shell.directory)
        self.set_filename(shell.filename)
        self.set_filters(shell.filters)
        self.set_selected_filter(shell.selected_filter)
    
    # The wx.FileDialog doesn't emit events for anything useful, 
    # so there is nothing which need be bound.

    #--------------------------------------------------------------------------
    # Shell Object Change Handlers
    #--------------------------------------------------------------------------
    def shell_mode_changed(self, mode):
        """ Update the dialog for the given mode of behavior.

        """
        self.set_mode(mode)

    def shell_multi_select_changed(self, multi_select):
        """ Update the dialog for the given multi select behavior.

        """
        self.set_multi_select(multi_select)

    def shell_directory_changed(self, directory):
        """ Update the dialog with the new working directory.

        """
        if not guard.guarded(self, 'setting_directory'):
            self.set_directory(directory)
                
    def shell_filename_changed(self, filename):
        """ Update the dialog with the new selected filename.

        """
        if not guard.guarded(self, 'setting_filename'):
            self.set_filename(filename)

    def shell_filters_changed(self, filters):
        """ Update the dialog with the new filename filters.

        """
        # Traits doens't fire off a change event if an Enum updates
        # because its underlying values have changed. So, we just 
        # set the selected filter manually here to cover that case.
        self.set_filters(filters)
        self.set_selected_filter(self.shell_obj.selected_filter)

    def shell_selected_filter_changed(self, selected_filter):
        """ Update the dialog with the new selected filter.

        """
        if not guard.guarded(self, 'setting_selected_filter'):
            self.set_selected_filter(selected_filter)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_mode(self, mode):
        """ Sets the mode behavior of the file dialog.

        """
        self.widget.SetWindowStyle(STYLE_MODE_MAP[mode])
    
    def set_multi_select(self, multi_select):
        """ Sets the multi select behavior of the file dialog.

        """
        widget = self.widget
        style = widget.GetWindowStyle()
        if style & wx.FD_OPEN:
            if multi_select:
                style |= wx.FD_MULTIPLE
            else:
                style &= ~wx.FD_MULTIPLE
            widget.SetWindowStyle(style)

    def set_directory(self, directory):
        """ Sets the current directory of the dialog.

        """
        self.widget.SetDirectory(directory)
    
    def set_filename(self, filename):
        """ Sets the selected filename in the dialog.

        """
        self.widget.SetFilename(filename)
    
    def set_filters(self, filters):
        """ Sets the name filters in the file dialog.

        """
        # This converts Qt's filter format, which is used by the shell
        # object, into wx's wildcard format.
        parts = []
        for filt in filters:
            parts.append(filt)
            match = FILTER_RE.search(filt)
            if match:
                filt = match.group(1)
            parts.append(u';'.join(filt.split()))
        wildcard = u'|'.join(parts)
        self.widget.SetWildcard(wildcard)
    
    def set_selected_filter(self, selected_filter):
        """ Sets the selected name filter in the file dialog.

        """
        try:
            idx = self.shell_obj.filters.index(selected_filter)
        except ValueError:
            return
        self.widget.SetFilterIndex(idx)

    def set_central_widget(self, widget):
        """ Overridden parent class method which makes central widget
        operations a no-op, since a file dialog has no children.

        """
        pass

    #--------------------------------------------------------------------------
    # Auxiliary Methods 
    #--------------------------------------------------------------------------
    def _handle_retcode(self, retcode):
        """ Overridden parent class method to update the shell object
        with new info once the dialog is closed.

        """
        widget = self.widget
        shell = self.shell_obj
        if retcode == wx.ID_OK:
            paths = widget.GetPaths()
            first_path = paths[0] if paths else u''
            with guard(self, 'setting_directory'):
                with guard(self, 'setting_filename'):
                    shell.directory, shell.filename = os.path.split(first_path)
                    shell._paths = paths
        idx = widget.GetFilterIndex()
        if idx < len(shell.filters):
            shell.selected_filter = shell.filters[idx]
        super(WXFileDialog, self)._handle_retcode(retcode)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .wx_control import WXControl

from ...components.toggle_control import AbstractTkToggleControl


class WXToggleControl(WXControl, AbstractTkToggleControl):
    """ A base class for wxPython toggle widgets.

    This class can serve as a base class for widgets that implement 
    toggle behavior such as CheckBox, RadioButton and checkable 
    PushButtons. It is not meant to be used directly. Subclasses should
    implement the 'create' and 'bind' methods.

    Furthermore, the toggled, pressed and released event that is 
    generated by the toolkit widget needs to be bound to the 
    on_toggled(), on_pressed() and on_released() methods.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def initialize(self):
        """ Initializes the attributes of the underlying control.

        """
        super(WXToggleControl, self).initialize()
        shell = self.shell_obj
        self.set_text(shell.text)
        self.set_checked(shell.checked)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_checked_changed(self, checked):
        """ The change handler for the 'checked' attribute.

        """
        self.set_checked(checked)

    def shell_text_changed(self, text):
        """ The change handler for the 'text' attribute.

        """
        self.set_text(text)
        # If the label of the control changes, its size hint has likely
        # updated and the layout system needs to be informed
        self.shell_obj.size_hint_updated()

    def on_toggled(self, event):
        """ The event handler for the toggled event.

        """
        shell = self.shell_obj
        shell.checked = self.widget.GetValue()
        shell._down = False
        shell.toggled()
        event.Skip()

    def on_pressed(self, event):
        """ The event handler for the pressed event.

        """
        shell = self.shell_obj
        shell._down = True
        shell.pressed()
        event.Skip()

    def on_released(self, event):
        """ The event handler for the released event.

        """
        shell = self.shell_obj
        if shell._down:
            shell._down = False
            shell.released()
        event.Skip()

    def set_text(self, text):
        """ Sets the widget's text with the provided value.

        """
        self.widget.SetLabel(text)

    def set_checked(self, checked):
        """ Sets the widget's checked state with the provided value.

        """
        self.widget.SetValue(checked)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant
from .. import operators

class TestQtLessLess(QtTestAssistant, operators.TestLessLess):
    """ TestSuite for the LessLess operator in Qt.

    """
    def get_text(self, widget):
        """ Returns the label text from the tookit widget of Label.

        """
        return widget.text()

    def get_checked(self, widget):
        """ Returns the label text from the tookit widget of CheckBox.

        """
        return widget.isChecked()



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import required_method
from .window import TestWindow


def skip_test(func):
    def closure(self, *args, **kwargs):
        self.skipTest('Skipped')
    return closure


class TestDialog(TestWindow):
    """ Logic for testing Dialogs.

    """

    def setUp(self):
        """ Set up Dialog tests.

        """

        enaml_source = """
enamldef MainView(Dialog):
    name = 'dialog'
    title = 'foo'
"""

        self.view = self.parse_and_create(enaml_source)
        self.component = self.component_by_name(self.view, 'dialog')
        self.widget = self.component.toolkit_widget
        # Don't actually show the dialog.
        self.disable_showing(self.widget)
        # (trait_name, value) log of all trait change events on the Dialog.
        self.event_log = []
        self.component.on_trait_change(self._append_event_handler, 'anytrait')

    def test_initial_active(self):
        """ Test the initial value of the active flag on the Dialog.

        """
        self.assertEquals(self.component.active, False)
    
    def test_result_value(self):
        """ Test the modification of the result value.

        """
        self.assertEquals(self.component.result, 'rejected')
        self.component.accept()
        self.assertEquals(self.component.result, 'accepted')
        self.component.reject()
        self.assertEquals(self.component.result, 'rejected')

    def test_show_close(self):
        """ Test the behavior when showing and closing the dialog.

        """
        self.component.abstract_obj.set_visible(True)
        # Compare sets because the order is unimportant.
        self.assertEquals(set(self.event_log), set([
            ('active', True),
            ('_active', True),
            ('opened', None),
        ]))
        self.event_log = []
        self.component.accept()
        self.assertEquals(set(self.event_log), set([
            ('active', False),
            ('_active', False),
            ('result', 'accepted'),
            ('_result', 'accepted'),
            ('closed', 'accepted'),
        ]))
        self.event_log = []
        self.component.abstract_obj.set_visible(True)
        self.assertEquals(set(self.event_log), set([
            ('active', True),
            ('_active', True),
            ('opened', None),
        ]))
        self.event_log = []
        self.component.reject()
        self.assertEquals(set(self.event_log), set([
            ('active', False),
            ('_active', False),
            ('result', 'rejected'),
            ('_result', 'rejected'),
            ('closed', 'rejected')
        ]))

    @required_method
    def disable_showing(self, widget):
        """ Disable the actual display of the dialog window.

        """
        pass

    def _append_event_handler(self, object, name, old, new):
        """ Append the trait change notification to the event log.

        """
        # XXX this test is fragile since it's collected all trait 
        # change events. Hence we need to filter out certain things
        # we don't care about. FIX THIS TEST!!!!!
        if name == '_relayout_pending':
            return
        self.event_log.append((name, new))




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
import wx
from wx.lib.splitter import MultiSplitterWindow

from .wx_constraints_widget import WXConstraintsWidget

from ...components.splitter import AbstractTkSplitter


_ORIENTATION_MAP = {
    'horizontal': wx.HORIZONTAL,
    'vertical': wx.VERTICAL,
}


class CustomSplitter(MultiSplitterWindow):
    """ A wx.lib.splitter.MultiSplitterWindow subclass that changes
    the behavior of resizing neighbors to be consisten with Qt.

    """
    def _OnMouse(self, event):
        """ Overriden parent class mouse event handler which fakes the
        state of the keyboard so that resize behavior is consistent
        between wx and Qt.

        """
        # We modify the mouse event to "fake" like the shift key is
        # always down. This causes the splitter to not adjust its 
        # neighbor when dragging the sash. This behavior is consistent
        # with Qt's behavior. This is not *the best* way to handle this,
        # but it's the easiest and quickest at the moment. The proper 
        # way would be to reimplement this method in its entirety and
        # allow the adjustNeighbor computation to be based on keyboard
        # state as well as attribute flags. 
        #
        # TODO implement this properly
        event.m_shiftDown = True
        return super(CustomSplitter, self)._OnMouse(event)


class WXSplitter(WXConstraintsWidget, AbstractTkSplitter):
    """ A Wx implementation of Splitter.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QSplitter control.

        """
        self.widget = CustomSplitter(parent)

    def initialize(self):
        """ Intializes the widget with the attributes of this instance.

        """
        super(WXSplitter, self).initialize()
        shell = self.shell_obj
        self.set_orientation(shell.orientation)
        self.set_live_drag(shell.live_drag)
        self.update_children()

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_orientation_changed(self, orientation):
        """ Update the orientation of the widget.

        """
        self.set_orientation(orientation)

    def shell_live_drag_changed(self, live_drag):
        """ The change handler for the 'live_drag' attribute of the
        shell object.

        """
        self.set_live_drag(live_drag)

    def shell_splitter_children_changed(self, children):
        """ The change handler for the 'layout_children' attribute of 
        the shell object.

        """
        self.update_children()
    
    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def set_orientation(self, orientation):
        """ Update the orientation of the splitter.

        """
        wx_orientation = _ORIENTATION_MAP[orientation]
        self.widget.SetOrientation(wx_orientation)

    def set_live_drag(self, live_drag):
        """ Updates the drag state of the splitter.

        """
        if live_drag:
            self.widget.WindowStyle |= wx.SP_LIVE_UPDATE
        else:
            self.widget.WindowStyle &= ~wx.SP_LIVE_UPDATE
            
    def update_children(self):
        """ Update the splitter's children with the current children.

        """
        shell = self.shell_obj
        widget = self.widget
        # XXX using private _windows attribute. Is there no way
        # to query the splitter for it's windows?
        for widget_child in widget._windows:
            widget.DetachWindow(widget_child)

        for child in shell.splitter_children:
            child_widget = child.toolkit_widget
            child_widget.Reparent(widget)
            widget.AppendWindow(child_widget)

    def size_hint(self):
        """ Return a size hint for the widget.

        """
        # XXX We're punting here for now. What we need to do is compute
        # the combined size hints for the children using the algorithm
        # implementation in QSplitter::sizeHint
        return (296, 172)

    def set_splitter_sizes(self, sizes):
        """ Set the initial sizes for the children.

        """
        # XXX We're punting here for now. We need to do is implement
        # QSplitter::setSizes
        return




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt.QtCore import QSize
from .qt.QtGui import QWidget, QLayout, QIcon
from .qt_widget_component import QtWidgetComponent

from ...components.window import AbstractTkWindow


class QtWindowLayout(QLayout):
    """ A QLayout subclass which can have at most one layout item. This
    layout item is expanded to fit the allowable space, regardless of its
    size policy settings. This is similar to how central widgets behave 
    in a QMainWindow.

    The class is designed for use by QtWindow/QtDialog, other uses are at 
    the user's own risk.

    """
    def __init__(self, *args, **kwargs):
        super(QtWindowLayout, self).__init__(*args, **kwargs)
        self._layout_item = None
    
    def addItem(self, item):
        """ A virtual method implementation which sets the layout item
        in the layout. Any old item will be overridden.

        """
        self._layout_item = item
        self.update()

    def count(self):
        """ A virtual method implementation which returns 0 if no layout
        item is supplied, or 1 if there is a current layout item.

        """
        return 0 if self._layout_item is None else 1

    def itemAt(self, idx):
        """ A virtual method implementation which returns the layout item 
        for the given index or None if one does not exist.

        """
        if idx == 0:
            return self._layout_item

    def takeAt(self, idx):
        """ A virtual method implementation which removes and returns the
        item at the given index or None if one does not exist.

        """
        if idx == 0:
            res = self._layout_item
            self._layout_item = None
            return res
    
    def sizeHint(self):
        """ A reimplemented method to return a proper size hint for the
        layout, and hence the Window.

        """
        return QSize(600, 100)

    def setGeometry(self, rect):
        """ A reimplemented method which sets the geometry of the managed
        widget to fill the given rect.

        """
        super(QtWindowLayout, self).setGeometry(rect)
        item = self._layout_item
        if item is not None:
            item.widget().setGeometry(rect)


class QtWindow(QtWidgetComponent, AbstractTkWindow):
    """ A Qt4 implementation of a Window.
     
    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying Qt widget.

        """
        self.widget = QWidget(parent)
        self.widget.setLayout(QtWindowLayout())

    def initialize(self):
        """ Intializes the attributes on the underlying Qt widget.

        """
        super(QtWindow, self).initialize()
        shell = self.shell_obj
        self.set_title(shell.title)
        self.set_icon(shell.icon)
        self.set_central_widget(shell.central_widget)

    #--------------------------------------------------------------------------
    # Abstract Toolkit Implementation
    #--------------------------------------------------------------------------
    def maximize(self):
        """ Maximizes the window to fill the screen.

        """
        self.widget.showMaximized()
            
    def minimize(self):
        """ Minimizes the window to the task bar.

        """
        self.widget.showMinimized()
            
    def restore(self):
        """ Restores the window after it has been minimized or maximized.

        """
        self.widget.showNormal()

    #--------------------------------------------------------------------------
    # Shell Object Change Handlers
    #--------------------------------------------------------------------------
    def shell_title_changed(self, title):
        """ The change handler for the 'title' attribute on the shell
        object.

        """
        self.set_title(title)

    def shell_icon_changed(self, icon):
        """ The change handler for the 'icon' attribute on the shell
        object.

        """
        self.set_icon(icon)

    def shell_central_widget_changed(self, central_widget):
        """ The change handler for the 'central_widget' attribute on 
        the shell object.

        """
        self.set_central_widget(central_widget)
    
    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_central_widget(self, central_widget):
        """ Sets the central widget in the window with the given value.

        """
        # It's possible for the central widget component to be None.
        # This must be allowed since the central widget may be generated
        # by an Include component, in which case it will not exist 
        # during initialization. However, we must have a central widget
        # for the Window, and so we just fill it with a dummy widget.
        if central_widget is None:
            child_widget = QWidget()
        else:
            child_widget = central_widget.toolkit_widget
        self.widget.layout().addWidget(child_widget)

    def set_icon(self, icon):
        """ Sets the icon of the underlying widget.

        """
        if icon is None:
            qicon = QIcon()
        else:
            qicon = icon.as_QIcon()
        self.widget.setWindowIcon(qicon)

    def set_title(self, title):
        """ Sets the title of the underlying widget.

        """
        self.widget.setWindowTitle(title)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from enaml.backends.qt.qt import QtGui, QtCore

from enaml.backends.qt.qt_slider import (HOR_TICK_POS_MAP,
                                         VERT_TICK_POS_MAP)

from .qt_test_assistant import QtTestAssistant
from .. import slider

# A map from Qt constants for horizontal or vertical orientation to Enaml enums.
ORIENTATION_MAP = {QtCore.Qt.Horizontal: 'horizontal',
                   QtCore.Qt.Vertical: 'vertical'}


# Map test event actions to the Qt Slider event signals
EVENT_MAP = {slider.TestEvents.PRESSED: 'sliderPressed',
             slider.TestEvents.RELEASED: 'sliderReleased'}

# Map test event actions to the Qt Slider actions
ACTION_MAP ={slider.TestEvents.HOME: QtGui.QAbstractSlider.SliderToMinimum,
             slider.TestEvents.END: QtGui.QAbstractSlider.SliderToMaximum,
             slider.TestEvents.STEP_UP: QtGui.QAbstractSlider.SliderSingleStepAdd,
             slider.TestEvents.STEP_DOWN: QtGui.QAbstractSlider.SliderSingleStepSub,
             slider.TestEvents.PAGE_UP: QtGui.QAbstractSlider.SliderPageStepAdd,
             slider.TestEvents.PAGE_DOWN: QtGui.QAbstractSlider.SliderPageStepSub}

class TestQtSlider(QtTestAssistant, slider.TestSlider):
    """ QtLabel tests. """

    def get_value(self, widget):
        """ Get a slider's position.

        """
        return widget.value()

    def get_minimum(self, widget):
        """ Get the Slider's minimum value.

        """
        return widget.minimum()

    def get_maximum(self, widget):
        """ Get the Slider's maximum value.

        """
        return widget.maximum()

    def get_tick_interval(self, widget):
        """ Get the Slider's tick_interval value.

        """
        return widget.tickInterval()

    def get_tick_position(self, widget):
        """ Get the Slider's tick position style.

        """
        value = widget.tickPosition()
        orientation = self.get_orientation(widget)
        if orientation == 'vertical':
            result = VERT_TICK_POS_MAP[value]
        else:
            result = HOR_TICK_POS_MAP[value]
        return result

    def get_orientation(self, widget):
        """ Get the Slider's orientation.

        """
        value = widget.orientation()
        return ORIENTATION_MAP[value]

    def get_single_step(self, widget):
        """ Get the Slider's single step value.

        """
        return widget.singleStep()

    def get_page_step(self, widget):
        """ Get the Slider's page step value.

        """
        return widget.pageStep()

    def get_tracking(self, widget):
        """ Get the Slider's tracking status.

        """
        return widget.hasTracking()

    def send_event(self, widget, event):
        """ Send an event to the Slider programmatically.

        Arguments
        ---------
        widget :
            The widget to send the event to.

        event :
            The desired event to be proccessed.

        """
        if event in ACTION_MAP:
            widget.triggerAction(ACTION_MAP[event])
        elif event in EVENT_MAP:
            getattr(widget, EVENT_MAP[event]).emit()
        else:
            raise NotImplementedError('Test event is not Implemented')




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import unittest
import inspect
import types

from enaml import default_toolkit
from enaml.core.parser import parse
from enaml.core.enaml_compiler import EnamlCompiler


def required_method(function_object):
    """ Decorator for required methods.

    The decorator wraps an *empty* method to raise an NotImplementedError
    with an appropriate error message.

    """
    def proxy_function(self, *args, **kwargs):
        function_name = function_object.__name__
        msg = ("Method '{0}' needs to be implemented for the '{1}' test case".\
                format(function_name, self))
        raise NotImplementedError(msg)
    return proxy_function


class EnamlTestCase(unittest.TestCase):
    """ Base class for testing Enaml object widgets.

    This class provide utility methods and assertion functions to help
    the testing of enaml components

    """
    # default toolkit to use for the enaml source parsing
    toolkit = default_toolkit()

    def component_by_name(self, component, name):
        """ Find an item in the view with a given name. The component
        should have the name set its 'name' attribute.

        Arguments
        ---------
        component : BaseComponent
            The enaml based BaseComponent object.

        name : str
            The name of the enaml component to find in the tree.

        Returns
        -------
            The corresponding component or None.

        """
        return component.find_by_name(name)

    def parse_and_create(self, source, **kwargs):
        """ Parses and compiles the source. The source should have a
        component defined with the name 'MainView'. 

        Arguments
        ---------
        source : str
            The enaml source file

        kwargs : dict
            The default attribute values to pass to the component.

        """
        enaml_ast = parse(source)
        enaml_module = types.ModuleType('__tests__')
        ns = enaml_module.__dict__
        code = EnamlCompiler.compile(enaml_ast, '__enaml_tests__')

        toolkit = self.toolkit

        with toolkit:
            exec code in ns
            view = ns['MainView']
            cmpnt = view(**kwargs)

        toolkit.app.initialize()
        self.app = toolkit.app.app_object()
        cmpnt.setup()
        return cmpnt

    def assertEnamlInSync(self, component, attribute_name, value):
        """ Verify that the requested attribute is properly set

        The method compares the attribute value in the Enaml object and
        check if it is synchronized with the toolkit widget. The component
        attribute is retrieved directly while the widget value is retrieved
        through a call to a method function in the test case.

        Arguments
        ---------
        component : enaml.widgets.component.Component
            The Enaml component to check.

        attribute_name : str
            The string name of the Enaml attribute to check.

        value :
            The expected value.

        .. note:: It is expected that the user has defined an appropriate
            method get_<attribute_name>(widget) or the extentded version
            get_<attribute_name>(component, widget) in the current test
            case. The extended signature is commonly used because additional
            information on the component's attributes is required to return
            a sensible result (e.g. the component uses Converters to set
            and retrieve the value of the attribute). The assert method
            The get methods can raise assertion errors when it is not
            possible to retrieve a sensible value for the attribute.

        """
        widget = component.toolkit_widget
        enaml_value = getattr(component, attribute_name)
        widget_method = getattr(self, 'get_' + attribute_name)

        try:
            inspect.getcallargs(widget_method, widget)
        except TypeError:
            widget_value = widget_method(component, widget)
        else:
            widget_value = widget_method(widget)

        self.assertEqual(value, enaml_value)
        self.assertEqual(value, widget_value)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .wx_test_assistant import WXTestAssistant, skip_nonwindows
from .. import html


@skip_nonwindows
class TestWxHtml(WXTestAssistant, html.TestHtml):
    """ WXHtml tests. """

    def get_source(self, widget):
        """ Get the source of an Html widget.

        """
        return widget.ToText()




#-----------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import os
import sys


#------------------------------------------------------------------------------
# Import Helper
#------------------------------------------------------------------------------
def imports():
    """ Lazily imports and returns an enaml imports context.

    """
    from .core.import_hooks import imports
    return imports()


#------------------------------------------------------------------------------
# Toolkit Factory Functions
#------------------------------------------------------------------------------
#: The private storage for the optional default toolkit function which 
#: overrides that which is computed from environment variables.
_default_toolkit_func = None


def set_default_toolkit_func(func):
    """ Set the default toolkit function to the given callable.

    Parameters
    ----------
    func : callable
        A callable object which takes no arguments and returns an
        instance of Toolkit.
    
    """
    global _default_toolkit_func
    _default_toolkit_func = func


def reset_default_toolkit_func():
    """ Reset the default toolkit func such that the default toolkit
    is computed from environment variables.

    """
    global _default_toolkit_func
    _default_toolkit_func = None


def default_toolkit():
    """ Creates an returns the default toolkit object based on the user's
    current ETS_TOOLKIT environment variables, or the default toolkit
    function supplied via the set_default_toolkit_func function.

    """
    tk_func = _default_toolkit_func
    if tk_func is not None:
        return tk_func()

    # Accepts forms such as 'qt4' and 'qt4.agg' to allow for kiva backends
    toolkit = os.environ.get('ETS_TOOLKIT', 'qt').lower().split('.')[0]

    if toolkit == 'qt' or toolkit == 'qt4':
        return qt_toolkit()

    if toolkit == 'wx':
        return wx_toolkit()

    raise ValueError('Invalid Toolkit: %s' % toolkit)


def qt_toolkit():
    """ Creates and return a toolkit object for the Qt backend.

    """
    from .core.operators import OPERATORS
    from .core.toolkit import Toolkit
    from .components.constructors import CONSTRUCTORS
    from .layout.layout_helpers import LAYOUT_HELPERS
    from .backends.qt.constructors import QT_CONSTRUCTORS
    from .backends.qt.qt_application import QtApplication
    from .backends.qt.noncomponents.toolkit_items import TOOLKIT_ITEMS

    toolkit = Toolkit(QT_CONSTRUCTORS)
    toolkit.update(TOOLKIT_ITEMS)
    toolkit.update(CONSTRUCTORS)
    toolkit.update(OPERATORS)
    toolkit.update(LAYOUT_HELPERS)
    toolkit.app = QtApplication()

    return toolkit


def wx_toolkit():
    """ Creates and return a toolkit object for the Wx backend.

    """
    from .core.operators import OPERATORS
    from .core.toolkit import Toolkit
    from .components.constructors import CONSTRUCTORS
    from .layout.layout_helpers import LAYOUT_HELPERS
    from .backends.wx.constructors import WX_CONSTRUCTORS
    from .backends.wx.wx_application import WXApplication
    from .backends.wx.noncomponents.toolkit_items import TOOLKIT_ITEMS

    toolkit = Toolkit(WX_CONSTRUCTORS)
    toolkit.update(TOOLKIT_ITEMS)
    toolkit.update(CONSTRUCTORS)
    toolkit.update(OPERATORS)
    toolkit.update(LAYOUT_HELPERS)
    toolkit.app = WXApplication()

    return toolkit


#------------------------------------------------------------------------------
# Test Helpers
#------------------------------------------------------------------------------
def test_collector():
    """ Discover and collect tests for the Enaml Package.

        .. note :: addapted from the unittest2
    """
    from unittest import TestLoader

    # import __main__ triggers code re-execution
    __main__ = sys.modules['__main__']
    setupDir = os.path.abspath(os.path.dirname(__main__.__file__))

    return TestLoader().discover(setupDir)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" A ModelIndex class implemented in Cython for speed.

"""
cdef class ModelIndex:
    """ ModelIndexes are used to navigate an AbstractItemModel.

    A ModelIndex is a lightweight object and should be created on the
    fly, rather than attempting some form of caching. The user will
    not normally need to create indexes manually. Instead, use the
    create_index method on an AbstractItemModel.

    """
    cdef int _row, _column
    cdef object _model, _context 

    def __init__(self, int row, int column, context, model):
        """ Construct a model index.

        Arguments
        ---------
        row : int
            The row index represented by this index

        column : int
            The column index represented by this index

        model : AbstractItemModel
            The model in which this index is active. This is typically
            supplied by the create_index method of the AbstractItemModel.

        context : object
            A user supplied object that aids in navigating the user's
            model.

        """
        self._row = row
        self._column = column
        self._context = context
        self._model = model

    property row:
        
        def __get__(self):
            return self._row

    property column:
        
        def __get__(self):
            return self._column

    property context:

        def __get__(self):
            return self._context

    property model:

        def __get__(self):
            return self._model

    def __richcmp_(self, _other, int op):
        """ Performs a rich comparison operation on this index with another 
        index. The results of comparing this object with a non-ModelIndex 
        always returns False and therefore represents undefined behavior.

        """
        if not isinstance(_other, ModelIndex):
            return False
        
        cdef tuple this_tuple, other_tuple
        cdef ModelIndex other = _other

        this_tuple = (self._row, self._column, self._context, self._model)
        other_tuple = (other._row, other._column, other._context, other._model)

        # ==
        if op == 2:
            return this_tuple == other_tuple

        # !=
        if op == 3:
            return this_tuple != other_tuple

        # <
        if op == 0:
            return this_tuple < other_tuple
        
        # >
        if op == 4:
            return this_tuple > other_tuple
        
        # <=
        if op == 1:
            return this_tuple <= other_tuple
        
        # >=
        if op == 5:
            return this_tuple >= other_tuple
            
        return False
    
    def __repr__(self):
        msg = 'ModelIndex(row=%s, col=%s, context=%s, model=%s)'
        return msg % (self._row, self._column, self._context, self._model)
    
    def __str__(self):
        return self.__repr__()

    def parent(self):
        """ Returns the parent ModelIndex of this index.

        """
        return self._model.parent(self)
    
    def sibling(self, int row, int column):
        """ Returns the sibling ModelIndex of this index for the given
        row and column indexes.

        """
        cdef object model = self._model
        if self._row == row and self._column == column:
            return self
        return model.index(row, column, model.parent(self))
    
    def child(self, int row, int column):
        """ Returns the child ModelIndex of this index for the given
        row and column indexes.

        """
        return self._model.index(row, column, self)

    def flags(self):
        """ Returns the flags for this index.

        """
        return self._model.flags(self)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import EnamlTestCase, required_method


class TestRadioButton(EnamlTestCase):
    """ Logic for testing radio buttons.

    Tooklit testcases need to provide the following methods

    Abstract Methods
    ----------------
    get_value(self, button)
        Get the checked state of a radio button.

    get_text(self, button)
        Get the label of a button.

    """
    def setUp(self):
        """ Finalise set up before the Radio button tests

        """

        label_1 = 'Label 1'
        label_2 = 'Label 2'

        enaml_source = """
enamldef MainView(MainWindow):
    Container:
        RadioButton:
            name = 'radio1'
            text = '{0}'
            checked = True
        RadioButton:
            name = 'radio2'
            text = '{1}'
""".format(label_1, label_2)

        self.events = []
        self.view = self.parse_and_create(enaml_source)
        self.radio1 = self.component_by_name(self.view, 'radio1')
        self.widget1 = self.radio1.toolkit_widget
        self.radio2 = self.component_by_name(self.view, 'radio2')
        self.widget2 = self.radio2.toolkit_widget

    def test_initial_value(self):
        """ Test the initial checked state of the radio buttons.

        """
        widget1_value = self.get_value(self.widget1)
        self.assertTrue(widget1_value)
        self.assertEqual(self.radio1.checked, widget1_value)

        widget2_value = self.get_value(self.widget2)
        self.assertFalse(widget2_value)
        self.assertEqual(self.radio2.checked, widget2_value)

    def test_initial_labels(self):
        """ Test that the toolkit widget's label reflects the Enaml text.

        """
        widget1_label = self.get_text(self.widget1)
        self.assertEqual(widget1_label, 'Label 1')
        self.assertEqual(self.radio1.text, widget1_label)

        widget2_label = self.get_text(self.widget2)
        self.assertEqual(widget2_label, 'Label 2')
        self.assertEqual(self.radio2.text, widget2_label)

    def test_change_label(self):
        """ Change the label of a RadioButton at the Enaml level.

        """
        new_label = 'new_label'
        self.radio2.text = new_label

        widget2_text = self.get_text(self.widget2)
        self.assertEqual(new_label, widget2_text)
        self.assertEqual(self.radio2.text, widget2_text)

    def test_set_checked(self):
        """ Test setting the value of a radio button in a group.

        """
        # Select the second button.
        self.radio2.checked = True

        widget2_value = self.get_value(self.widget2)
        self.assertTrue(widget2_value)
        self.assertEqual(self.radio2.checked, widget2_value)

        # Select the first button to deselect second.
        self.radio1.checked = True

        widget2_value = self.get_value(self.widget2)
        self.assertFalse(widget2_value)
        self.assertEqual(self.radio2.checked, widget2_value)

    def test_multiple_radio_buttons(self):
        """Test selecting one of a set radiobuttons. """

        # select second
        self.radio2.checked = True

        widget1_value = self.get_value(self.widget1)
        widget2_value = self.get_value(self.widget2)

        # The selected radio button is ofcourse aware (we did this)
        self.assertEqual(self.radio2.checked, widget2_value)

        # Both the wxwidgets know of the change!
        self.assertTrue(widget2_value)
        self.assertFalse(widget1_value)

        self.assertEqual(self.radio1.checked, widget1_value)

    #--------------------------------------------------------------------------
    # absrtact methods
    #--------------------------------------------------------------------------
    @required_method
    def get_value(self, button):
        """ Get the checked state of a radio button.

        """
        pass

    @required_method
    def get_text(self, button):
        """ Get the label of a button.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import (
    Instance, Either, Unicode, Property, List, cached_property,
)

from .action import Action
from .widget_component import WidgetComponent, AbstractTkWidgetComponent

from ..core.trait_types import EnamlEvent


class AbstractTkMenu(AbstractTkWidgetComponent):
    """ The abstract toolkit interface for a Menu object.

    """
    @abstractmethod
    def shell_title_changed(self, title):
        """ The change handler for the 'title' attribute on the shell
        object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_contents_changed(self, contents):
        """ The change handler for the list of contents on the
        shell object.

        """
        raise NotImplementedError
        
    @abstractmethod
    def popup(self, blocking=True):
        """ Create and show the menu as context or popup menu.

        Parameters
        ----------
        blocking : boolean, optional
            Whether or not the popup menu should be displayed in a 
            blocking context. If False, then this menu object must
            be stored on the heap or it will be immediately garbage
            collected. Defaults to True.

        """
        raise NotImplementedError


class Menu(WidgetComponent):
    """ A declarative Enaml Component which represents a menu in a 
    menu bar.

    """
    #: The title of the menu as displayed by a MenuBar
    title = Unicode

    #: An event emitted when the menu is about to be shown.
    about_to_show = EnamlEvent

    #: An event emitted when the menu is about to be hidden.
    about_to_hide = EnamlEvent

    #: A read-only cached property which holds the list of menu
    #: contents which are instances of Menu or Action.
    contents = Property(
        List(Either(Instance('Menu'), Instance(Action))), 
        depends_on='children',
    )

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkMenu)
    
    @cached_property
    def _get_contents(self):
        """ The property getter for the 'contents' attribute.

        """
        flt = lambda child: isinstance(child, (Menu, Action))
        return filter(flt, self.children)

    def popup(self, blocking=True, parent=None):
        """ Create and show the menu as context or popup menu.

        If the menu has not been initialized when this method is
        called, it will be setup and initialized automatically.

        Parameters
        ----------
        blocking : boolean, optional
            Whether or not the popup menu should be displayed in a 
            blocking context. If False, then this menu object must
            be stored on the heap or it will be immediately garbage
            collected. Defaults to True. This option may not be 
            supported on all toolkit backends. 
        
        parent : Toolkit widget or None, optional
            The parent on which to popup the menu. This only has an
            effect on menus which are created outside of the normal 
            Enaml heirarchy and are setup and initialized during the 
            call to this method. Even then, the popup menu will be a 
            top-level widget, but it's lifetime will be tied to that 
            of the parent.

        """
        if not self.initialized:
            self.setup(parent)
        self.abstract_obj.popup(blocking=blocking)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_constraints_widget import QtConstraintsWidget

from ...components.control import AbstractTkControl


class QtControl(QtConstraintsWidget, AbstractTkControl):
    """ A Qt4 implementation of Control.

    """
    pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_container import QtContainer

from ...components.form import AbstractTkForm


class QtForm(QtContainer, AbstractTkForm):
    """ A Qt4 implementation of Container.

    """
    # The QtContainer implementation is enough.
    pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------

import sys
from unittest import skipUnless

import wx

from enaml import wx_toolkit


skip_nonwindows = skipUnless(sys.platform.startswith('win'),
    "The wx backend is only supported on Windows.")


class WXTestAssistant(object):
    """ Assistant class for testing wx based components.

    This class is to be used as a mixing aling with the base enaml test case
    class for the components tests of the wx backend. It sets the correct
    toolkit attribute and provides some useful methods to mock events in
    wx.

    """

    toolkit = wx_toolkit()

    def process_wx_events(self, app):
        """ Process posted wxPython events.

        Tests that require resize, move and show/hide or generic event
        response take place need to call this method.

        """
        app.ProcessPendingEvents()

    def send_wx_event(self, widget, event_type):
        """ Send a wxPython widget an event (e.g., EVT_BUTTON).

        """
        event = wx.PyCommandEvent(event_type.typeId, widget.GetId())
        widget.GetEventHandler().ProcessEvent(event)
        self.process_wx_events(self.app)

    def send_wx_mouse_event(self, widget, event_type, position=None):
        """ Send a wxPython widget an mouse event (e.g., EVT_BUTTON).

        """
        event = wx.MouseEvent(event_type.typeId)
        if position is None:
            event.m_x = 0
            event.m_y = 0
        else:
            event.m_x = position.x
            event.m_y = position.y
        widget.GetEventHandler().ProcessEvent(event)
        self.process_wx_events(self.app)



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import unittest

from enaml.validation import CoercingValidator


@unittest.skip('Skipping Validation Tests')
class TestCoercingValidator(unittest.TestCase):

    def test_all_valid(self):
        v = CoercingValidator()
        for item in (u'foo', u'12', u'foo12', u'#$@$@#$%!@#'):
            self.assertEqual(v.validate(item), v.ACCEPTABLE)
    
    def test_thru_convert(self):
        v = CoercingValidator()
        for item in (u'foo', u'12', u'foo12', u'#$@$@#$%!@#'):
            self.assertEqual(v.convert(item), item)
    
    def test_thru_format(self):
        v = CoercingValidator()
        for item in (u'foo', u'12', u'foo12', u'#$@$@#$%!@#'):
            self.assertEqual(v.format(item), item)
    
    def test_default_format(self):
        v = CoercingValidator()
        for item in (42, 42.0, object, object()):
            self.assertEqual(v.format(item), unicode(item))
    
    def test_convert_func(self):
        v = CoercingValidator(converter=int)
        for item in (u'42', u'456', u'789'):
            self.assertEqual(v.convert(item), int(item))

    def test_coerce_func(self):
        v = CoercingValidator(coercer=int)
        for item in (u'42', u'456', u'789'):
            self.assertEqual(v.convert(item), int(item))
        
    def test_format_func(self):
        fmt = lambda val: unicode(val) + u'foo'
        v = CoercingValidator(formatter=fmt)
        for item in (u'42', u'45.6', u'78.9'):
            self.assertEqual(v.format(item), item + u'foo')

    def test_normalize(self):
        v = CoercingValidator(converter=float, coercer=int)
        self.assertEqual(v.normalize(u'12.3'), u'12')

    def test_convert_coerce_func(self):
        v = CoercingValidator(converter=float, coercer=int)
        for item, res in ((u'42', 42), (u'67.98', 67)):
            self.assertEqual(v.convert(item), res)

    def test_has_locale(self):
        v = CoercingValidator()
        self.assertTrue(v.locale is not None)

    def test_locale_raises(self):
        v = CoercingValidator()
        with self.assertRaises(TypeError):
            v.locale = 12

    def test_locale_allow_none(self):
        v = CoercingValidator()
        v.locale = None




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance, Property, cached_property

from .base_selection_model import BaseSelectionModel
from .control import Control, AbstractTkControl

from ..core.trait_types import EnamlEvent
from ..item_models.abstract_item_model import AbstractItemModel
from ..item_models.model_index import ModelIndex


class AbstractTkItemView(AbstractTkControl):
    """ The abstract toolkit ItemView interface.

    """
    @abstractmethod
    def shell_item_model_changed(self, model):
        """ The change handler for the 'item_model' attribute on the
        shell object.

        """
        raise NotImplementedError


# XXX rename this to ItemView
class AbstractItemView(Control):
    """ An abstract base class view that contains common logic for the
    ListView, TableView, and TreeView classes.
    
    """
    #: The AbstractItemModel instance being displayed by the view.
    item_model = Instance(AbstractItemModel)

    #: The selection model for this view. If more than one selection
    #: model is declared and exception will be raised.
    selection_model = Property(
        Instance(BaseSelectionModel), depends_on='children',
    )
    
    #: The ModelIndex that has just been activated by a user interaction,
    #: usually a double-click or an Enter keypress.
    activated = EnamlEvent(ModelIndex)

    #: The ModelIndex that has just been clicked.
    clicked = EnamlEvent(ModelIndex)

    #: The ModelIndex that has just been double-clicked.
    double_clicked = EnamlEvent(ModelIndex)

    #: Overridden parent class trait.
    abstract_obj = Instance(AbstractTkItemView)

    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_selection_model(self):
        """ The property getter for the 'selection_model' attribute. It
        creates a default selection model if one is not provided.

        """
        flt = lambda child: isinstance(child, BaseSelectionModel)
        sel_models = filter(flt, self.children)
        n_models = len(sel_models)
        if n_models == 0:
            res = BaseSelectionModel()
        elif n_models == 1:
            res = sel_models[0]
        else:
            msg = 'An ItemView can have exactly 1 SelectionModel. %s given.'
            raise ValueError(msg % n_models)
        return res




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_test_assistant import WXTestAssistant, skip_nonwindows

from .. import combo_box


@skip_nonwindows
class TestWXComboBox(WXTestAssistant, combo_box.TestComboBox):
    """ WXComboBox tests. 

    """
    def get_selected_text(self, widget):
        """ Get the current selected text of a combo box.

        """
        return widget.GetStringSelection()

    def get_item_text(self, widget, index):
        """ Get the text of a combo box item at a particular index.

        """
        return widget.GetString(index)

    def select_item(self, widget, index):
        """ Fire an event to simulate the selection of an item.

        """
        widget.SetSelection(index)
        self.send_wx_event(widget, wx.EVT_COMBOBOX)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# Copyright (c) 2010, Enthought Inc
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license.

#
# Author: Enthought Inc
# Description: Qt API selector. Can be used to switch between pyQt and PySide
#------------------------------------------------------------------------------

import os

def prepare_pyqt4():
    # Set PySide compatible APIs.
    import sip
    sip.setapi('QString', 2)
    sip.setapi('QVariant', 2)

qt_api = os.environ.get('QT_API')

if qt_api is None:
    try:
        import PySide
        qt_api = 'pyside'
        os.environ['QT_API'] = qt_api
    except ImportError:
        try:
            prepare_pyqt4()
            import PyQt4
            qt_api = 'pyqt'
            os.environ['QT_API'] = qt_api
        except ImportError:
            raise ImportError('Cannot import PySide or PyQt4')

elif qt_api == 'pyqt':
    prepare_pyqt4()

elif qt_api != 'pyside':
    raise RuntimeError("Invalid Qt API %r, valid values are: 'pyqt' or 'pyside'"
                       % qt_api)



#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod
import os

from traits.api import (
    Bool, Enum, List, Unicode, Instance, Property, cached_property
)

from .dialog import AbstractTkDialog, Dialog


class AbstractTkFileDialog(AbstractTkDialog):
    """ The abstract toolkit interface for a FileDialog.

    """
    @abstractmethod
    def shell_mode_changed(self, mode):
        """ Update the dialog for the given mode of behavior.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_multi_select_changed(self, multi_select):
        """ Update the dialog for the given multi select behavior.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_directory_changed(self, directory):
        """ Update the dialog with the new working directory.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_filename_changed(self, filename):
        """ Update the dialog with the new selected filename.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_filters_changed(self, filters):
        """ Update the dialog with the new filename filters.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_selected_filter_changed(self, selected_filter):
        """ Update the dialog with the new selected filter.

        """
        raise NotImplementedError


class FileDialog(Dialog):
    """ A dialog widget that allows the user to open/save files.

    """
    #: The mode of the dialog: 'open' or 'save'
    mode = Enum('open', 'save')

    #: Whether to allow selecting multiple files in 'open' mode. 
    multi_select = Bool(False)

    #: The current directory of the file dialog.
    directory = Unicode

    #: The file selected in the dialog.
    filename = Unicode

    #: A read-only property which returns the full path to the file,
    #: or the first file in the selection if multi_select is True.
    path = Property(Unicode, depends_on=['directory', 'filename'])

    #: A read-only property which returns a list of selected paths.
    paths = Property(List(Unicode), depends_on='_paths')

    #: The private internal storage for the 'paths' property. This is
    #: updated by the toolkit specific backends.
    _paths = List(Unicode)

    #: The string filters used to restrict the set of files.
    filters = List(Unicode)

    #: The selected filter from the list of filters.
    selected_filter = Enum(values='filters')

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkFileDialog)

    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_path(self):
        """ The property getter for the 'path' attribute.

        """
        return os.path.join(self.directory, self.filename)

    @cached_property
    def _get_paths(self):
        """ The property getter for the 'paths' attribute.

        """
        return self._paths
    
    #--------------------------------------------------------------------------
    # Parent Class Overrides 
    #--------------------------------------------------------------------------
    def add_subcomponent(self, component):
        """ An overriden parent class method which prevents subcomponents
        from being declared for a FileDialog instance.

        """
        msg = "Cannot add subcomponents to a FileDialog."
        raise ValueError(msg)




# -*- coding: utf-8 -*-
"""
    enamldoc
    ~~~~~~~~

    Automatically insert docstrings for enaml built-in and derived widgets and
    components, mirroring and borrowing heavily from the autodoc
    extension. Enaml widgets and components are Python objects that are
    imported in the enaml.imports() context

"""

import re
import inspect
from types import FunctionType, BuiltinFunctionType

from sphinx.application import ExtensionError
from sphinx.util.inspect import getargspec
from sphinx.ext.autodoc import Documenter
from sphinx.ext.autodoc import ModuleLevelDocumenter
from sphinx.ext.autodoc import DocstringSignatureMixin
from sphinx.ext.autodoc import ModuleDocumenter

from enaml_domain import EnamlDomain

from enthought.debug.api import called_from

from enaml.core.import_hooks import EnamlImporter
EnamlImporter.install()

from enaml.core.factory import EnamlDeclaration

class EnamlComponentDocumenter(ModuleLevelDocumenter):
    """ Enaml component documenter class

    The main purpose of this class is to be distinct from Python components and
    change the domain in the Documenter instance to 'enaml'

    """

    def __init__(self, directive, name, indent=u''):
        """ Need to override the parent __init__ so that we can set
        self.domain, which is an instance variable.

        """

        super(EnamlComponentDocumenter, self).__init__(directive, name, indent)
        self.domain = 'enaml'

    def check_module(self):
        """ For Enaml objects, the module name returned by
        self.object.__module__ points to the enaml parsing compiler, rather
        than the logical module name stored in self.modname. Therefore, this
        check will verify that the module points to
        enaml.parsing.enaml_compiler.

        Note - after a fix to correct the .__module__ and .__file__ attributes,
        this method will not be necessary and will need to be deleted.

        """

        #import pdb; pdb.set_trace()
        return self.get_attr(self.object, '__module__', None) == \
            'enaml.parsing.enaml_compiler'

    def format_signature(self):
        """ Thin method for debugging signature formatting.

        """

        super(EnamlComponentDocumenter, self).format_signature()


class EnamlDocstringSignatureMixin(DocstringSignatureMixin):
    """ Mixin for FunctionDocumenter and MethodDocumenter to provide the
    feature of reading the signature from the docstring.

    identical to the autodoc.DocstringSignatureMixin it subclasses and included
    here as an indicator of a possible future avenue for working with docstring
    signatures in enaml -- possibly to replace refactordoc.

    """

    pass


class EnamlDeclarationDocumenter(EnamlDocstringSignatureMixin,
                                 EnamlComponentDocumenter):
    """ Specialized Documenter subclass for Enaml declarations.

    """

    objtype = 'enaml_decl'
    member_order = 35

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, EnamlDeclaration)

    def format_args(self):
        """ Derivation is shown where normally arguments are shown in a
        function prototype.

        """

        ## import pdb; pdb.set_trace()
        return ' (derives from {0})'.format(self.object.__base__)

    def document_members(self, all_members=False):
        pass


## class EnamlDefnDocumenter(EnamlDocstringSignatureMixin,
##                           EnamlComponentDocumenter):
##     """ Specialized Documenter subclass for Enaml defns.

##     """

##     objtype = 'enaml_defn'
##     member_order = 32

##     @classmethod
##     def can_document_member(cls, member, membername, isattr, parent):
##         return isinstance(member, EnamlDefn)

##     def format_args(self):
##         ## if inspect.isbuiltin(self.object) or \
##         ##        inspect.ismethoddescriptor(self.object):
##         ##     # cannot introspect arguments of a C function or method
##         ##     return None
##         try:
##             argspec = getargspec(self.object.__func__)
##         except TypeError:
##             # if a class should be documented as function (yay duck
##             # typing) we try to use the constructor signature as function
##             # signature without the first argument.
##             try:
##                 argspec = getargspec(self.object.__new__)
##             except TypeError:
##                 argspec = getargspec(self.object.__init__)
##                 if argspec[0]:
##                     del argspec[0][0]
##         args = inspect.formatargspec(*argspec)
##         # escape backslashes for reST
##         args = args.replace('\\', '\\\\')
##         return args

##     def document_members(self, all_members=False):
##         """ We may want to use this in the future to include submembers of a
##         defn.

##         """
##         pass


class EnamlModuleDocumenter(ModuleDocumenter):
    """ Subclass of the module documenter for enaml autodocumenting.

    The main purpose of the subclass is to set the domain so that member
    directives are formed properly.

    """

    objtype = 'enaml_module'

    def __init__(self, directive, name, indent=u''):
        """ Need to override the parent __init__ so that we can set
        self.domain, which is an instance variable.

        """
        super(EnamlModuleDocumenter, self).__init__(directive, name, indent)
        self.domain = 'enaml'

    def get_object_members(self, want_all):
        """ Shim method for debugging """
        ## print "EnamlModuleDocumenter: self.options.members:\n {0}".format(
        ##     self.options.members)
        ## import pdb; pdb.set_trace()
        return super(EnamlModuleDocumenter, self).get_object_members(want_all)


def add_documenter(cls):
    """ Register a new Documenter.

    This autodoc function is overridden here solely for the sake of the proper
    error message.

    """
    if not issubclass(cls, Documenter):
        raise ExtensionError('enamldoc documenter %r must be a subclass '
                             'of autodoc.Documenter' % cls)
    AutoDirective._registry[cls.objtype] = cls


def setup(app):
    """ enamldoc extension setup function.

    The setup function is called by Sphinx when loading extensions.  app is the
    instance of the calling app, Sphinx in this case.

    enamldoc borrows heavily from autodoc, so additions to setup and
    configuration are minimal.

    """

    app.add_autodocumenter(EnamlModuleDocumenter)
    app.add_autodocumenter(EnamlDeclarationDocumenter)
    ## app.add_autodocumenter(EnamlDefnDocumenter)

    # import the Enaml domain into the sphinx app
    app.domains['enaml'] = EnamlDomain


    ## app.add_event('enamldoc-import-component')
    ## app.add_event('enamldoc-process-component')


class testcls:
    """ test doc string """

    def __getattr__(self, x):
        return x

    def __setattr__(self, x, y):
        """Attr setter."""
